//#include "php_sbb.h"
#include "sbphpcore.h"
#include "sbsystem.h"

#ifdef ZEND_WIN32
#  include <windows.h>
#endif

zend_class_entry *TSBPointer_ce_ptr = NULL;
zend_class_entry *TSBBaseStruct_ce_ptr = NULL;

zend_class_entry *TObject_ce_ptr = NULL;
zend_object_handlers TObject_handlers;

#ifdef SB_DEBUG
void SBOutputDebugString(const char *format, ...) 
{
	va_list args;
	va_start(args, format);

#	ifdef ZEND_WIN32
	fprintf(stderr, "[%d] ", GetTickCount());

	{
		char output_buf[1024];

		vsnprintf(output_buf, 1024, format, args);
		OutputDebugString(output_buf);
		OutputDebugString("\n");
	}
#	endif

	vfprintf(stderr, format, args);
	fprintf(stderr, "\n");
	va_end(args);
}
#	endif

#if PHP_VERSION_ID < 50600L
// from zend_constants.c v5.6
void zend_register_null_constant(const char *name, uint name_len, int flags, int module_number TSRMLS_DC)
{
	zend_constant c;

	ZVAL_NULL(&c.value);
	c.flags = flags;
	c.name = zend_strndup(name, name_len - 1);
	c.name_len = name_len;
	c.module_number = module_number;
	zend_register_constant(&c TSRMLS_CC);
}
#endif

SB_PHP_FUNCTION(SBUtils, BytesOfString)
{
	char *arg = NULL;
	int i, arg_len;

	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == SUCCESS)
	{
		array_init_size(return_value, arg_len);
		for (i = 0; i < arg_len; i++)
			add_next_index_long(return_value, (uint8_t)arg[i]);
	}
	else
	{
		SBErrorExpectsArguments("(string)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBUtils, StringOfBytes)
{
	zval *arr;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "a", &arr) == SUCCESS) 
	{
		HashTable *ht = HASH_OF(arr);
		HashPosition pos;
		zval **entry;

		int32_t sOutResult_len = zend_hash_num_elements(ht);
		uint8_t *sOutResult = emalloc(sOutResult_len + 1);
		ulong idx = 0;
		for (zend_hash_internal_pointer_reset_ex(ht, &pos);
			zend_hash_get_current_data_ex(ht, (void **)&entry, &pos) == SUCCESS;
			zend_hash_move_forward_ex(ht, &pos))
		{
			char *key;
			uint key_len;
			ulong index;

			if (zend_hash_get_current_key_ex(ht, &key, &key_len, &index, 0, &pos) != HASH_KEY_IS_LONG)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "String keys are not allowed in the byte array");				
				efree(sOutResult);
				RETURN_FALSE;
			}

			if (idx != index)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "The indexes are not sequential");
				efree(sOutResult);
				RETURN_FALSE;
			}

			if (IS_NULL == Z_TYPE_PP(entry))
			{
				sOutResult[idx] = 0;
			}
			else if (IS_LONG == Z_TYPE_PP(entry))
			{
				sOutResult[idx] = (uint8_t)Z_LVAL_PP(entry);
			}
			else
			{
				zval temp = **entry;
				zval_copy_ctor(&temp);
				convert_to_long(&temp);
				sOutResult[idx] = (uint8_t)Z_LVAL(temp);
				zval_dtor(&temp);
			}

			idx++;
		}

		sOutResult[sOutResult_len] = 0;
		RETURN_STRINGL(sOutResult, sOutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("(array of byte)" TSRMLS_CC);
	}
}

void SBInitArrayZValInfo(SBArrayZValInfo *info)
{
	(*info).data = NULL;
	(*info).len = 0;
	(*info).ownData = 0;
	(*info).dataType = IS_STRING;
}

void SBClearArrayZValInfo(SBArrayZValInfo *info)
{
	(*info).data = NULL;
	(*info).len = 0;
	(*info).ownData = 0;
	(*info).dataType = IS_NULL;
}

void SBFreeArrayZValInfo(SBArrayZValInfo *info)
{
	if ((*info).data && (*info).ownData)
		efree((*info).data);

	(*info).data = NULL;
	(*info).len = 0;
	(*info).ownData = 0;
	(*info).dataType = IS_NULL;
}

int SBGetByteArrayFromZVal(zval *arr, SBArrayZValInfo *info TSRMLS_DC)
{
	if (IS_STRING == Z_TYPE_P(arr))
	{
		(*info).data = Z_STRVAL_P(arr);
		(*info).len = Z_STRLEN_P(arr);
		(*info).ownData = 0;
		(*info).dataType = IS_STRING;
	}
	else if (IS_ARRAY == Z_TYPE_P(arr))
	{
		HashTable *ht = HASH_OF(arr);
		HashPosition pos;
		zval **entry;
		ulong idx = 0;

		(*info).len = zend_hash_num_elements(ht);
		(*info).data = emalloc((*info).len);
		(*info).ownData = 1;
		(*info).dataType = IS_ARRAY;
		for (zend_hash_internal_pointer_reset_ex(ht, &pos);
			zend_hash_get_current_data_ex(ht, (void **)&entry, &pos) == SUCCESS;
			zend_hash_move_forward_ex(ht, &pos))
		{
			char *key;
			uint key_len;
			ulong index;

			if (zend_hash_get_current_key_ex(ht, &key, &key_len, &index, 0, &pos) != HASH_KEY_IS_LONG)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "String keys are not allowed in the array");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (idx != index)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "The indexes are not sequential");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (IS_NULL == Z_TYPE_PP(entry))
			{
				((uint8_t *)(*info).data)[idx] = 0;
			}
			else if (IS_LONG == Z_TYPE_PP(entry))
			{
				((uint8_t *)(*info).data)[idx] = (uint8_t)Z_LVAL_PP(entry);
			}
			else
			{
				zval temp = **entry;
				zval_copy_ctor(&temp);
				convert_to_long(&temp);
				((uint8_t *)(*info).data)[idx] = (uint8_t)Z_LVAL(temp);
				zval_dtor(&temp);
			}

			idx++;
		}		
	}
	else
	{
		(*info).data = NULL;
		(*info).len = 0;
		(*info).ownData = 0;
		(*info).dataType = IS_NULL;
	}

	return 1;
}

void SBSetByteArrayToZValP(uint8_t * data, int32_t len, zval *arr)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, len);
	for (i = 0; i < len; i++)
		add_next_index_long(arr, data[i]);
}

void SBSetByteArrayToZVal(SBArrayZValInfo *info, zval *arr)
{
	if (IS_ARRAY == (*info).dataType)
	{
		int i;
		zval_dtor(arr);
		array_init_size(arr, (*info).len);
		for (i = 0; i < (*info).len; i++)
			add_next_index_long(arr, ((uint8_t *)(*info).data)[i]);

		SBFreeArrayZValInfo(info);
	}
	else
	{
		if ((*info).ownData || (IS_STRING != (*info).dataType))
		{
			zval_dtor(arr);
			ZVAL_STRINGL(arr, (*info).data, (*info).len, 0);
		}
		else
		{
			// we had enough space in the original variable
			if (Z_STRLEN_P(arr) > (*info).len)
				((char *)(*info).data)[(*info).len] = 0;

			Z_STRLEN_P(arr) = (*info).len;
		}

		SBClearArrayZValInfo(info);
	}
}

void SBDetachDataFromZValAndResize(zval *arr, SBArrayZValInfo *info TSRMLS_DC)
{
	if ((*info).ownData)
	{
		(*info).data = erealloc((*info).data, (*info).len + 1);
	}
	else
	{
		if (((*info).dataType != IS_STRING) || (Z_TYPE_P(arr) != IS_STRING))
		{
			php_error_docref(NULL TSRMLS_CC, E_WARNING, "SBDetachDataFromZValAndResize: invalid data type");
			return;
		}

		zval_dtor(arr);
		Z_TYPE_P(arr) = IS_NULL;
		(*info).data = emalloc((*info).len + 1);
		(*info).ownData = 1;
	}
}

void SBResizeArrayZVal(SBArrayZValInfo *info TSRMLS_DC)
{
	if ((*info).ownData)
		(*info).data = erealloc((*info).data, (*info).len); 
	else
	{
		(*info).data = emalloc((*info).len);
		(*info).ownData = 1;
	}
}

int SBGetInt16ArrayFromZVal(zval *arr, SBArrayZValInfo *info TSRMLS_DC)
{
	if (IS_ARRAY == Z_TYPE_P(arr))
	{
		HashTable *ht = HASH_OF(arr);
		HashPosition pos;
		zval **entry;
		ulong idx = 0;

		(*info).len = zend_hash_num_elements(ht) << 1;
		(*info).data = emalloc((*info).len);
		(*info).ownData = 1;
		(*info).dataType = IS_ARRAY;
		for (zend_hash_internal_pointer_reset_ex(ht, &pos);
			zend_hash_get_current_data_ex(ht, (void **)&entry, &pos) == SUCCESS;
			zend_hash_move_forward_ex(ht, &pos))
		{
			char *key;
			uint key_len;
			ulong index;

			if (zend_hash_get_current_key_ex(ht, &key, &key_len, &index, 0, &pos) != HASH_KEY_IS_LONG)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "String keys are not allowed in the array");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (idx != index)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "The indexes are not sequential");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (IS_NULL == Z_TYPE_PP(entry))
			{
				((int16_t *)(*info).data)[idx] = 0;
			}
			else if (IS_LONG == Z_TYPE_PP(entry))
			{
				((int16_t *)(*info).data)[idx] = (int16_t)Z_LVAL_PP(entry);
			}
			else
			{
				zval temp = **entry;
				zval_copy_ctor(&temp);
				convert_to_long(&temp);
				((int16_t *)(*info).data)[idx] = (int16_t)Z_LVAL(temp);
				zval_dtor(&temp);
			}

			idx++;
		}
	}
	else
	{
		(*info).data = NULL;
		(*info).len = 0;
		(*info).ownData = 0;
		(*info).dataType = IS_NULL;
	}

	return 1;
}

void SBSetInt16ArrayToZValP(int16_t * data, int32_t len, zval *arr)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, len >> 1);
	for (i = 0; i < (len >> 1); i++)
		add_next_index_long(arr, data[i]);
}

void SBSetInt16ArrayToZVal(SBArrayZValInfo *info, zval *arr)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, (*info).len >> 1);
	for (i = 0; i < ((*info).len >> 1); i++)
		add_next_index_long(arr, ((int16_t *)(*info).data)[i]);

	SBFreeArrayZValInfo(info);
}

int SBGetInt32ArrayFromZVal(zval *arr, SBArrayZValInfo *info TSRMLS_DC)
{
	if (IS_ARRAY == Z_TYPE_P(arr))
	{
		HashTable *ht = HASH_OF(arr);
		HashPosition pos;
		zval **entry;
		ulong idx = 0;

		(*info).len = zend_hash_num_elements(ht) << 2;
		(*info).data = emalloc((*info).len);
		(*info).ownData = 1;
		(*info).dataType = IS_ARRAY;
		for (zend_hash_internal_pointer_reset_ex(ht, &pos);
			zend_hash_get_current_data_ex(ht, (void **)&entry, &pos) == SUCCESS;
			zend_hash_move_forward_ex(ht, &pos))
		{
			char *key;
			uint key_len;
			ulong index;

			if (zend_hash_get_current_key_ex(ht, &key, &key_len, &index, 0, &pos) != HASH_KEY_IS_LONG)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "String keys are not allowed in the array");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (idx != index)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "The indexes are not sequential");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (IS_NULL == Z_TYPE_PP(entry))
			{
				((int32_t *)(*info).data)[idx] = 0;
			}
			else if (IS_LONG == Z_TYPE_PP(entry))
			{
				((int32_t *)(*info).data)[idx] = (int32_t)Z_LVAL_PP(entry);
			}
			else
			{
				zval temp = **entry;
				zval_copy_ctor(&temp);
				convert_to_long(&temp);
				((int32_t *)(*info).data)[idx] = (int32_t)Z_LVAL(temp);
				zval_dtor(&temp);
			}

			idx++;
		}
	}
	else
	{
		(*info).data = NULL;
		(*info).len = 0;
		(*info).ownData = 0;
		(*info).dataType = IS_NULL;
	}

	return 1;
}

void SBSetInt32ArrayToZValP(int32_t * data, int32_t len, zval *arr)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, len >> 2);
	for (i = 0; i < (len >> 2); i++)
		add_next_index_long(arr, data[i]);
}

void SBSetInt32ArrayToZVal(SBArrayZValInfo *info, zval *arr)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, (*info).len >> 2);
	for (i = 0; i < ((*info).len >> 2); i++)
		add_next_index_long(arr, ((int32_t *)(*info).data)[i]);

	SBFreeArrayZValInfo(info);
}

int SBGetUInt32ArrayFromZVal(zval *arr, SBArrayZValInfo *info TSRMLS_DC)
{
	if (IS_ARRAY == Z_TYPE_P(arr))
	{
		HashTable *ht = HASH_OF(arr);
		HashPosition pos;
		zval **entry;
		ulong idx = 0;

		(*info).len = zend_hash_num_elements(ht) << 2;
		(*info).data = emalloc((*info).len);
		(*info).ownData = 1;
		(*info).dataType = IS_ARRAY;
		for (zend_hash_internal_pointer_reset_ex(ht, &pos);
			zend_hash_get_current_data_ex(ht, (void **)&entry, &pos) == SUCCESS;
			zend_hash_move_forward_ex(ht, &pos))
		{
			char *key;
			uint key_len;
			ulong index;

			if (zend_hash_get_current_key_ex(ht, &key, &key_len, &index, 0, &pos) != HASH_KEY_IS_LONG)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "String keys are not allowed in the array");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (idx != index)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "The indexes are not sequential");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (IS_NULL == Z_TYPE_PP(entry))
			{
				((uint32_t *)(*info).data)[idx] = 0;
			}
			else if (IS_LONG == Z_TYPE_PP(entry))
			{
				((uint32_t *)(*info).data)[idx] = (uint32_t)Z_LVAL_PP(entry);
			}
			else
			{
				zval temp = **entry;
				zval_copy_ctor(&temp);
				convert_to_long(&temp);
				((uint32_t *)(*info).data)[idx] = (uint32_t)Z_LVAL(temp);
				zval_dtor(&temp);
			}

			idx++;
		}
	}
	else
	{
		(*info).data = NULL;
		(*info).len = 0;
		(*info).ownData = 0;
		(*info).dataType = IS_NULL;
	}

	return 1;
}

void SBSetUInt32ArrayToZValP(uint32_t * data, int32_t len, zval *arr)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, len >> 2);
	for (i = 0; i < (len >> 2); i++)
		add_next_index_long(arr, data[i]);
}

void SBSetUInt32ArrayToZVal(SBArrayZValInfo *info, zval *arr)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, (*info).len >> 2);
	for (i = 0; i < ((*info).len >> 2); i++)
		add_next_index_long(arr, ((uint32_t *)(*info).data)[i]);

	SBFreeArrayZValInfo(info);
}

int SBGetInt64ArrayFromZVal(zval *arr, SBArrayZValInfo *info TSRMLS_DC)
{
	if (IS_ARRAY == Z_TYPE_P(arr))
	{
		HashTable *ht = HASH_OF(arr);
		HashPosition pos;
		zval **entry;
		ulong idx = 0;

		(*info).len = zend_hash_num_elements(ht) << 3;
		(*info).data = emalloc((*info).len);
		(*info).ownData = 1;
		(*info).dataType = IS_ARRAY;
		for (zend_hash_internal_pointer_reset_ex(ht, &pos);
			zend_hash_get_current_data_ex(ht, (void **)&entry, &pos) == SUCCESS;
			zend_hash_move_forward_ex(ht, &pos))
		{
			char *key;
			uint key_len;
			ulong index;

			if (zend_hash_get_current_key_ex(ht, &key, &key_len, &index, 0, &pos) != HASH_KEY_IS_LONG)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "String keys are not allowed in the array");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (idx != index)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "The indexes are not sequential");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (IS_NULL == Z_TYPE_PP(entry))
			{
				((int64_t *)(*info).data)[idx] = 0;
			}
			else if (IS_LONG == Z_TYPE_PP(entry))
			{
				((int64_t *)(*info).data)[idx] = (int64_t)Z_LVAL_PP(entry);
			}
			else
			{
				zval temp = **entry;
				zval_copy_ctor(&temp);
				convert_to_long(&temp);
				((int64_t *)(*info).data)[idx] = (int64_t)Z_LVAL(temp);
				zval_dtor(&temp);
			}

			idx++;
		}
	}
	else
	{
		(*info).data = NULL;
		(*info).len = 0;
		(*info).ownData = 0;
		(*info).dataType = IS_NULL;
	}

	return 1;
}

void SBSetInt64ArrayToZValP(int64_t * data, int32_t len, zval *arr)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, len >> 3);
	for (i = 0; i < (len >> 3); i++)
		add_next_index_long(arr, (long)data[i]);
}

void SBSetInt64ArrayToZVal(SBArrayZValInfo *info, zval *arr)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, (*info).len >> 3);
	for (i = 0; i < ((*info).len >> 3); i++)
		add_next_index_long(arr, (long)((int64_t *)(*info).data)[i]);

	SBFreeArrayZValInfo(info);
}

int SBGetObjectArrayFromZVal(zval *arr, zend_class_entry *obj_ce, SBArrayZValInfo *info TSRMLS_DC)
{
	if (IS_ARRAY == Z_TYPE_P(arr))
	{
		HashTable *ht = HASH_OF(arr);
		HashPosition pos;
		zval **entry;
		ulong idx = 0;

		(*info).len = zend_hash_num_elements(ht) * sizeof(TElClassHandle);
		(*info).data = emalloc((*info).len);
		(*info).ownData = 1;
		(*info).dataType = IS_ARRAY;
		for (zend_hash_internal_pointer_reset_ex(ht, &pos);
			zend_hash_get_current_data_ex(ht, (void **)&entry, &pos) == SUCCESS;
			zend_hash_move_forward_ex(ht, &pos))
		{
			char *key;
			uint key_len;
			ulong index;

			if (zend_hash_get_current_key_ex(ht, &key, &key_len, &index, 0, &pos) != HASH_KEY_IS_LONG)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "String keys are not allowed in the array");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (idx != index)
			{
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "The indexes are not sequential");
				SBFreeArrayZValInfo(info);
				return 0;
			}

			if (IS_OBJECT == Z_TYPE_PP(entry))
			{
				((TElClassHandle *)(*info).data)[idx] = SBGetObjectHandleCE(*entry, obj_ce TSRMLS_CC);
			}
			else
			{
				((TElClassHandle *)(*info).data)[idx] = SB_NULL_HANDLE;
			}

			idx++;
		}
	}
	else
	{
		(*info).data = NULL;
		(*info).len = 0;
		(*info).ownData = 0;
		(*info).dataType = IS_NULL;
	}

	return 1;
}

void SBSetObjectArrayToZValP(TElClassHandle * data, int32_t len, zend_class_entry *obj_ce, zval *arr TSRMLS_DC)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, len / sizeof(TElClassHandle));
	for (i = 0; i < (int)(len / sizeof(TElClassHandle)); i++)
	{
		zval * obj;
		MAKE_STD_ZVAL(obj);
		SBInitObject(obj, obj_ce, data[i] TSRMLS_CC);
		add_next_index_zval(arr, obj);
	}
}

void SBSetObjectArrayToZVal(SBArrayZValInfo *info, zend_class_entry *obj_ce, zval *arr TSRMLS_DC)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, (*info).len / sizeof(TElClassHandle));
	for (i = 0; i < (int)((*info).len / sizeof(TElClassHandle)); i++)
	{
		zval * obj;
		MAKE_STD_ZVAL(obj);
		SBInitObject(obj, obj_ce, ((TElClassHandle *)(*info).data)[i] TSRMLS_CC);
		add_next_index_zval(arr, obj);
	}

	SBFreeArrayZValInfo(info);
}

void SBSetStructArrayToZValP(void * data, int32_t len, size_t size, zend_class_entry *obj_ce, zval *arr TSRMLS_DC)
{
	int i;
	zval_dtor(arr);
	array_init_size(arr, (uint)(len / size));
	for (i = 0; i < (int)(len / size); i++)
	{
		zval * obj;
		MAKE_STD_ZVAL(obj);
		SBInitStructObject(obj, obj_ce, data, size TSRMLS_CC);
		add_next_index_zval(arr, obj);
		data = (void *)((uint8_t *)data + size);
	}
}

static zend_always_inline int is_derived_class(zend_class_entry *child_class, zend_class_entry *parent_class)
{
	child_class = child_class->parent;
	while (child_class) {
		if (child_class == parent_class) {
			return 1;
		}
		child_class = child_class->parent;
	}

	return 0;
}

int SBClassEntryIsPointerOrObject(zend_class_entry *ce)
{
	if ((ce == TSBPointer_ce_ptr) || (ce == TObject_ce_ptr))
		return 1;

	return is_derived_class(ce, TObject_ce_ptr);
}

void * SBGetPointerValue(zval *ptr TSRMLS_DC)
{
	if ((Z_TYPE_P(ptr) == IS_OBJECT) && (Z_OBJCE_P(ptr) == TSBPointer_ce_ptr))
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(ptr TSRMLS_CC);
		return intern->data;
	}
	else
		return NULL;
}

int SBGetPointerFromZVal(zval *ptr, SBPointerZValInfo *info TSRMLS_DC)
{
	if ((Z_TYPE_P(ptr) == IS_OBJECT) && (Z_OBJCE_P(ptr) == TSBPointer_ce_ptr))
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(ptr TSRMLS_CC);
		(*info).data = intern->data;
		(*info).len = (int)intern->len;
		(*info).ownData = 0;
		(*info).dataType = IS_OBJECT;
		return 1;
	}
	else if ((Z_TYPE_P(ptr) == IS_OBJECT) && ((Z_OBJCE_P(ptr) == TObject_ce_ptr) || (is_derived_class(Z_OBJCE_P(ptr), TObject_ce_ptr))))
	{
		(*info).data = (void *)SBGetObjectHandle(ptr TSRMLS_CC);
		(*info).len = 0;
		(*info).ownData = 0;
		(*info).dataType = IS_OBJECT;
		return 1;
	}
	else
		return SBGetByteArrayFromZVal(ptr, info TSRMLS_CC);
}

void SBSetPointerToZVal(SBPointerZValInfo *info, zval *ptr)
{
	if (IS_OBJECT == (*info).dataType)
	{
		// php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(ptr TSRMLS_CC);
		// nothing to do
	}
	else
		SBSetByteArrayToZVal(info, ptr);
}

void SBInitPointerObject(zval *ptr, void * data TSRMLS_DC)
{
	if (data != NULL)
	{
		php_sbb_pointer * intern;
		object_init_ex(ptr, TSBPointer_ce_ptr);
		intern = (php_sbb_pointer *)zend_object_store_get_object(ptr TSRMLS_CC);
		intern->data = data;
		intern->len = 0;
		intern->ownData = 0;
	}
	else
	{
		INIT_ZVAL(*ptr);
		ZVAL_NULL(ptr);
	}
}

void SBInitPointerZValInfo(zval *ptr, SBPointerZValInfo *info TSRMLS_DC)
{
	if ((Z_TYPE_P(ptr) == IS_OBJECT) && (Z_OBJCE_P(ptr) == TSBPointer_ce_ptr))
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(ptr TSRMLS_CC);
		(*info).data = intern->data;
		(*info).len = (int)intern->len;
		(*info).ownData = 0;
		(*info).dataType = IS_OBJECT;
	}
	else
		SBInitArrayZValInfo(info);
}

void SBFreePointerZValInfo(SBPointerZValInfo *info)
{
	SBFreeArrayZValInfo(info);
}

#pragma region TObject

static void PHP_TObject_Free(void *object TSRMLS_DC)
{
	php_sbb_object *intern = (php_sbb_object *)object;
	
	if (!intern) {
		return;
	}

	if (intern->_OwnHandle)
		TObject_Free(intern->_Handle);
	intern->_Handle = SB_NULL_HANDLE;

	zend_object_std_dtor(&intern->zo TSRMLS_CC);
	
	efree(intern);
}

zend_object_value PHP_TObject_New(zend_class_entry *class_type TSRMLS_DC)
{
	zend_object_value retval;
	php_sbb_object *intern;

	intern = emalloc(sizeof(php_sbb_object));
	memset(intern, 0, sizeof(php_sbb_object));

	zend_object_std_init(&intern->zo, class_type TSRMLS_CC);
#if PHP_VERSION_ID >= 50400L
	object_properties_init(&intern->zo, class_type);
#endif

	retval.handle = zend_objects_store_put(intern, (zend_objects_store_dtor_t)zend_objects_destroy_object, (zend_objects_free_object_storage_t)PHP_TObject_Free, NULL TSRMLS_CC);
	retval.handlers = &TObject_handlers;

	return retval;
}

TElClassHandle SBGetObjectHandle(zval *obj TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT))
		return SB_NULL_HANDLE;
	else
	{
		php_sbb_object * intern = (php_sbb_object *)zend_object_store_get_object(obj TSRMLS_CC);
		return intern->_Handle;
	}
}

TElClassHandle SBGetObjectHandleCE(zval *obj, zend_class_entry *obj_ce TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT) || ((Z_OBJCE_P(obj) != obj_ce) && (!is_derived_class(Z_OBJCE_P(obj), obj_ce))))
		return SB_NULL_HANDLE;
	else
	{
		php_sbb_object * intern = (php_sbb_object *)zend_object_store_get_object(obj TSRMLS_CC);
		return intern->_Handle;
	}
}

void SBSetObjectHandle(zval *obj, const TElClassHandle handle, const TElOwnHandle ownHandle TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT))
		return;
	else
	{
		php_sbb_object * intern = (php_sbb_object *)zend_object_store_get_object(obj TSRMLS_CC);
		if (intern->_OwnHandle)
			TObject_Free(intern->_Handle);

		intern->_Handle = handle;
		intern->_OwnHandle = ownHandle;
	}
}

void SBInitObject(zval *obj, zend_class_entry *obj_ce, const TElClassHandle handle TSRMLS_DC)
{
	if (handle != SB_NULL_HANDLE)
	{
		php_sbb_object * intern;
		object_init_ex(obj, obj_ce);
		//SBSetObjectHandle(obj, handle, ohFalse TSRMLS_CC);
		intern = (php_sbb_object *)zend_object_store_get_object(obj TSRMLS_CC);
		intern->_Handle = handle;
		intern->_OwnHandle = ohFalse;
	}
	else
	{
		INIT_ZVAL(*obj);
		ZVAL_NULL(obj);
	}
}

void SBUpdateObjectHandle(zval *obj, const TElClassHandle handle TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT))
		return;
	else
	{
		php_sbb_object * intern = (php_sbb_object *)zend_object_store_get_object(obj TSRMLS_CC);
		intern->_Handle = handle;
	}
}

void SBDetachObjectHandle(zval *obj TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT))
		return;
	else
	{
		php_sbb_object * intern = (php_sbb_object *)zend_object_store_get_object(obj TSRMLS_CC);
		intern->_Handle = SB_NULL_HANDLE;
	}
}

void SBDetachObjectHandleIfUniqueZVal(zval *obj TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT))
		return;
	else
	{
		if (Z_REFCOUNT_P(obj) == 1)
		{
			php_sbb_object * intern = (php_sbb_object *)zend_object_store_get_object(obj TSRMLS_CC);
			intern->_Handle = SB_NULL_HANDLE;
		}
	}
}

SB_PHP_METHOD(TObject, __clone)
{
	/* Should never be executable */
	zend_throw_exception(NULL, "Cannot clone object using __clone()", 0 TSRMLS_CC);
}

SB_PHP_METHOD(TObject, __get)
{
	char *arg = NULL;
	int arg_len;
	if ((zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == FAILURE) || (arg_len < 1))
		return;
	else
	{
		zval *object = getThis();
		zval *result_ptr = NULL;
		char *name = emalloc(arg_len + 5);
		memcpy(name + 4, arg, arg_len);
		name[0] = 'g';
		name[1] = 'e';
		name[2] = 't';
		name[3] = '_';
		name[arg_len + 4] = 0;

		zend_call_method(&object, NULL, NULL, name, arg_len + 4, &result_ptr, 0, NULL, NULL TSRMLS_CC);

		efree(name);
		if (result_ptr) {
			RETVAL_ZVAL_FAST(result_ptr);
			zval_ptr_dtor(&result_ptr);
		}
	}
}

SB_PHP_METHOD(TObject, __set)
{
	zval *zValue = NULL;
	char *arg = NULL;
	int arg_len;
	if ((zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sz", &arg, &arg_len, &zValue) == FAILURE) || (arg_len < 1))
		return;
	else
	{
		zval *object = getThis();
		char *name = emalloc(arg_len + 5);
		memcpy(name + 4, arg, arg_len);
		name[0] = 's';
		name[1] = 'e';
		name[2] = 't';
		name[3] = '_';
		name[arg_len + 4] = 0;
		zend_call_method(&object, NULL, NULL, name, arg_len + 4, NULL, 1, zValue, NULL TSRMLS_CC);
		efree(name);
	}
}

SB_PHP_METHOD(TObject, __isset)
{
	zend_error(E_COMPILE_ERROR, "Cannot use isset()/empty() on the result of a function/property call (you can use \"null !== func()\" instead)");
}

SB_PHP_METHOD(TObject, __call)
{
	// hide _Inst name
}

SB_PHP_METHOD(TObject, detachHandle)
{
	if (ZEND_NUM_ARGS() > 0)
	{
		ZEND_WRONG_PARAM_COUNT();
	}
	else
	{
		SBDetachObjectHandle(getThis() TSRMLS_CC);
	}
}

SB_PHP_METHOD(TObject, getOwnHandle)
{
	if (ZEND_NUM_ARGS() > 0)
	{
		ZEND_WRONG_PARAM_COUNT();
	}
	else
	{
		zval *obj = getThis();
		if (NULL == obj)
		{
			RETURN_FALSE;
		}
		else
		{
			php_sbb_object * intern = (php_sbb_object *)zend_object_store_get_object(obj TSRMLS_CC);
			RETURN_BOOL(intern->_OwnHandle);
		}
	}
}

SB_PHP_METHOD(TObject, setOwnHandle)
{
	zend_bool bOwn;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "b", &bOwn) == SUCCESS)
	{
		zval *obj = getThis();
		if (NULL == obj)
		{
			RETURN_FALSE;
		}
		else
		{
			php_sbb_object * intern = (php_sbb_object *)zend_object_store_get_object(obj TSRMLS_CC);
			intern->_OwnHandle = bOwn;
		}
	}
	else
	{
		SBErrorExpectsArguments("(bool)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TObject, castTo)
{
	char *sName = NULL;
	int sName_len = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "|s", &sName, &sName_len) == SUCCESS)
	{
		zval *obj = getThis();
		TElClassHandle handle;
		zend_class_entry **pce;
		char Buf[256];

		if (NULL == obj)
		{
			RETURN_NULL();
		}

		handle = SBGetObjectHandle(obj TSRMLS_CC);
		if (!handle)
		{
			RETURN_NULL();
		}

		if (0 == sName_len)
		{
			int32_t szName = 255;
			SBCheckError(TObject_ClassName_1(handle, &Buf[0], &szName) TSRMLS_CC);
			Buf[szName] = 0;
			sName = &Buf[0];
			sName_len = (int)szName;
			if (0 == sName_len)
			{
				RETURN_NULL();
			}
		}

		if (zend_lookup_class(sName, sName_len, &pce TSRMLS_CC) == FAILURE)
			pce = NULL;

		if (pce && ((*pce == TObject_ce_ptr) || (is_derived_class(*pce, TObject_ce_ptr))))
		{
			object_init_ex(return_value, *pce);
			SBSetObjectHandle(return_value, handle, ohFalse TSRMLS_CC);
		}
		else
		{
			zend_error(E_WARNING, "Failed to cast to type %s", sName);
			RETURN_NULL();
		}
	}
	else
	{
		SBErrorExpectsArguments("(|string)" TSRMLS_CC);
	}
}

int PHP_TObject_compare_objects(zval *object1, zval *object2 TSRMLS_DC)
{
	php_sbb_object *intern1, *intern2;
	intern1 = (php_sbb_object *)zend_object_store_get_object(object1 TSRMLS_CC);
	intern2 = (php_sbb_object *)zend_object_store_get_object(object2 TSRMLS_CC);
	
	if (intern1->_Handle == intern2->_Handle)
		return 0;
	else
		return 1;
}

#pragma endregion 

#pragma region SBException and error handling

static zend_class_entry *SBException_ce_ptr;
static zend_class_entry *SBInsufficientBufferSizeException_ce_ptr;
static zend_class_entry *SBInvalidBufferSizeException_ce_ptr;
static zend_class_entry *SBInvalidParameterException_ce_ptr;
static zend_class_entry *SBInternalException_ce_ptr;

void SBExceptionInit(zval *object, long code TSRMLS_DC)
{
	int32_t errorCode = SBGetLastErrorCode();
	int32_t supplErrorCode = SBGetLastSupplErrorCode();
	char *errorName = NULL;
	char *errorMsg = NULL;
	char *errorStackTrace = NULL;
	char *message;
	char *str;
	int errorName_len = 0;
	int errorMsg_len = 0;
	int errorStackTrace_len = 0;
	int message_len;
	uint32_t err;
	
	zend_update_property_long(SBException_ce_ptr, object, "errorcode", sizeof("errorcode") - 1, (long)errorCode TSRMLS_CC);
	zend_update_property_long(SBException_ce_ptr, object, "supplerrorcode", sizeof("supplerrorcode") - 1, (long)supplErrorCode TSRMLS_CC);

	err = SBGetLastErrorNameA(NULL, &errorName_len);
	if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == err)
	{
		errorName = emalloc(errorName_len + 1);
		err = SBGetLastErrorNameA(errorName, &errorName_len);
		errorName[errorName_len] = 0;
	}

	if (SB_ERROR_OK == err)
	{
		zend_update_property_stringl(SBException_ce_ptr, object, "errorname", sizeof("errorname") - 1, errorName, errorName_len TSRMLS_CC);
	}
	else
	{
		if (errorName)
			efree(errorName);
	}

	err = SBGetLastErrorMessageA(NULL, &errorMsg_len);
	if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == err)
	{
		errorMsg = emalloc(errorMsg_len + 1);
		err = SBGetLastErrorMessageA(errorMsg, &errorMsg_len);
		errorMsg[errorMsg_len] = 0;
	}

	if (SB_ERROR_OK == err)
	{
		zend_update_property_stringl(SBException_ce_ptr, object, "errormessage", sizeof("errormessage") - 1, errorMsg, errorMsg_len TSRMLS_CC);
	}
	else
	{
		if (errorMsg)
			efree(errorMsg);
	}

	err = SBGetLastErrorStackTraceA(NULL, &errorStackTrace_len);
	if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == err)
	{
		errorStackTrace = emalloc(errorStackTrace_len + 1);
		err = SBGetLastErrorStackTraceA(errorStackTrace, &errorStackTrace_len);
		errorStackTrace[errorStackTrace_len] = 0;
	}

	if (SB_ERROR_OK == err)
	{
		zend_update_property_stringl(SBException_ce_ptr, object, "errorstacktrace", sizeof("errorstacktrace") - 1, errorStackTrace, errorStackTrace_len TSRMLS_CC);
	}

	if (errorStackTrace)
		efree(errorStackTrace);

	if (SB_ERROR_RUNTIME == code)
		str = "runtime";
	else
		str = "library";

	message_len = errorName_len + errorMsg_len + 130;
	message = emalloc(message_len);

	if (supplErrorCode != 0)
		message_len = slprintf(message, message_len, "SecureBlackbox %s exception: %s(Message: '%s', ErrorCode: 0x%08x, SupplErrorCode: 0x%08x)",
			str, errorName, errorMsg, errorCode, supplErrorCode);
	else
		message_len = slprintf(message, message_len, "SecureBlackbox %s exception: %s(Message: '%s', ErrorCode: 0x%08x)",
			str, errorName, errorMsg, errorCode);

	zend_update_property_stringl(SBException_ce_ptr, object, "message", sizeof("message") - 1, message, message_len TSRMLS_CC);

	if (message)
		efree(message);
	if (errorName)
		efree(errorName);
	if (errorMsg)
		efree(errorMsg);
}


SB_PHP_METHOD(SBException, __construct)
{
	long code = 0;
	zval *previous = NULL;

	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l|O!", &code, &previous, zend_exception_get_default(TSRMLS_C)) == FAILURE) 
	{
		zend_error(E_ERROR, "Wrong parameters for SBException(long $code [, Exception $previous = NULL])");
	}
	else
	{
		zval *object = getThis();
		if (code)
		{
			zend_update_property_long(SBException_ce_ptr, object, "code", sizeof("code") - 1, code TSRMLS_CC);
		}

		if (previous)
		{
			zend_update_property(SBException_ce_ptr, object, "previous", sizeof("previous") - 1, previous TSRMLS_CC);
		}

		SBExceptionInit(object, code TSRMLS_CC);
	}
}

static void SBException_get_entry(zval *object, char *name, int name_len, zval *return_value TSRMLS_DC) 
{
	zval *value;

	value = zend_read_property(SBException_ce_ptr, object, name, name_len, 0 TSRMLS_CC);

	*return_value = *value;
	zval_copy_ctor(return_value);
	INIT_PZVAL(return_value);
}

SB_PHP_METHOD(SBException, getErrorCode)
{
	if (zend_parse_parameters_none() == FAILURE) 
			return;

	SBException_get_entry(getThis(), "errorcode", sizeof("errorcode") - 1, return_value TSRMLS_CC);
}

SB_PHP_METHOD(SBException, getSupplErrorCode)
{
	if (zend_parse_parameters_none() == FAILURE)
		return;

	SBException_get_entry(getThis(), "supplerrorcode", sizeof("supplerrorcode") - 1, return_value TSRMLS_CC);
}

SB_PHP_METHOD(SBException, getErrorName)
{
	if (zend_parse_parameters_none() == FAILURE) 
		return;

	SBException_get_entry(getThis(), "errorname", sizeof("errorname") - 1, return_value TSRMLS_CC);
}

SB_PHP_METHOD(SBException, getErrorMessage)
{
	if (zend_parse_parameters_none() == FAILURE) 
		return;

	SBException_get_entry(getThis(), "errormessage", sizeof("errormessage") - 1, return_value TSRMLS_CC);
}

SB_PHP_METHOD(SBException, getErrorStackTrace)
{
	if (zend_parse_parameters_none() == FAILURE) 
		return;

	SBException_get_entry(getThis(), "errorstacktrace", sizeof("errorstacktrace") - 1, return_value TSRMLS_CC);
}

ZEND_BEGIN_ARG_INFO_EX(arginfo_SBException___construct, 0, 0, 0)
	ZEND_ARG_INFO(0, code)
	ZEND_ARG_INFO(0, previous)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_SBException_get_entry, 0, 0, 0)
ZEND_END_ARG_INFO()

static zend_function_entry SBException_methods[] = {
	PHP_ME(SBException, __construct, arginfo_SBException___construct, ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
	PHP_ME(SBException, getErrorCode, arginfo_SBException_get_entry, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(SBException, getSupplErrorCode, arginfo_SBException_get_entry, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(SBException, getErrorName, arginfo_SBException_get_entry, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(SBException, getErrorMessage, arginfo_SBException_get_entry, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(SBException, getErrorStackTrace, arginfo_SBException_get_entry, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_FE_END
};

zval * SBThrowException(uint32_t error TSRMLS_DC)
{
	zval *ex = NULL;
	if ((SB_ERROR_RUNTIME == error) || (SB_ERROR_LIBRARY == error))
	{
		ex = zend_throw_exception(SBException_ce_ptr, NULL, (long)error TSRMLS_CC);
		SBExceptionInit(ex, error TSRMLS_CC);
	}
	else if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == error)
		ex = zend_throw_exception(SBInsufficientBufferSizeException_ce_ptr, "Insufficient Buffer Size", 0 TSRMLS_CC);
	else if (SB_ERROR_INVALID_BUFFER_SIZE == error)
		ex = zend_throw_exception(SBInvalidBufferSizeException_ce_ptr, "Invalid Buffer Size", 0 TSRMLS_CC);
	else if (SB_ERROR_INVALID_PARAMETER == error)
		ex = zend_throw_exception(SBInvalidParameterException_ce_ptr, "Insufficient Parameter", 0 TSRMLS_CC);
	else
		ex = zend_throw_exception(SBInternalException_ce_ptr, "Internal error", 0 TSRMLS_CC);

	return ex;
}

void SBCheckError(uint32_t error TSRMLS_DC)
{
	if (SB_ERROR_OK != error)
	{
		SBThrowException(error TSRMLS_CC);
	}
}

void SBErrorExpectsArguments(const char *str TSRMLS_DC)
{
	zend_error(E_WARNING, "%s() expect arguments %s", get_active_function_name(TSRMLS_C), str);
}

#pragma endregion

#pragma region TSBPointer

void SBSetPointer(zval *obj, void *ptr TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT) || (Z_OBJCE_P(obj) != TSBPointer_ce_ptr))
		return;
	else
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(obj TSRMLS_CC);
		if (intern->data && intern->ownData)
			efree(intern->data);

		intern->data = ptr;
		intern->len = 0;
		intern->ownData = 0;
	}
}

static zend_object_handlers TSBPointer_handlers;

static void PHP_TSBPointer_Free(void *object TSRMLS_DC)
{
	php_sbb_pointer *intern = (php_sbb_pointer *)object;

	if (!intern) 
		return;

	if (intern->data && intern->ownData)
		efree(intern->data);
	intern->data = NULL;
	intern->len = 0;
	intern->ownData = 0;

	zend_object_std_dtor(&intern->zo TSRMLS_CC);

	efree(intern);
}

static zend_object_value PHP_TSBPointer_New(zend_class_entry *class_type TSRMLS_DC)
{
	zend_object_value retval;
	php_sbb_pointer *intern;

	intern = emalloc(sizeof(php_sbb_pointer));
	memset(&intern->zo, 0, sizeof(php_sbb_pointer));

	zend_object_std_init(&intern->zo, class_type TSRMLS_CC);
#if PHP_VERSION_ID >= 50400L
	object_properties_init(&intern->zo, class_type);
#endif

	retval.handle = zend_objects_store_put(intern, (zend_objects_store_dtor_t)zend_objects_destroy_object, (zend_objects_free_object_storage_t)PHP_TSBPointer_Free, NULL TSRMLS_CC);
	retval.handlers = &TSBPointer_handlers;

	return retval;
}

SB_PHP_METHOD(TSBPointer, __construct)
{
}

SB_PHP_METHOD(TSBPointer, __clone)
{
	/* Should never be executable */
	zend_throw_exception(NULL, "Cannot clone object using __clone()", 0 TSRMLS_CC);
}

SB_PHP_METHOD(TSBPointer, getData)
{
	long lSize;
	if (ZEND_NUM_ARGS() == 0)
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(getThis() TSRMLS_CC);
		RETURN_STRINGL(intern->data, (int)intern->len, 1);
	}
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &lSize) == SUCCESS)
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if (!intern->data)
		{
			RETURN_EMPTY_STRING();
		}
		else if (intern->ownData)
		{
			RETURN_STRINGL(intern->data, (int)(((size_t)lSize > intern->len) ? intern->len : lSize), 1);
		}
		else
		{
			RETURN_STRINGL(intern->data, (int)lSize, 1);
		}
	}
	else
	{
		SBErrorExpectsArguments("() or (long)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TSBPointer, setData)
{
	zval *zaData;
	SBArrayZValInfo aiData;
	zend_bool allocData = 1;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "z|b", &zaData, &allocData) == SUCCESS) && ((IS_STRING == Z_TYPE_P(zaData)) || (IS_ARRAY == Z_TYPE_P(zaData)) || (IS_NULL == Z_TYPE_P(zaData))))
	{
		php_sbb_pointer * intern;
		if (!SBGetByteArrayFromZVal(zaData, &aiData TSRMLS_CC)) RETURN_FALSE;
		
		intern = (php_sbb_pointer *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if (aiData.len > 0)
		{
			if (intern->data && intern->ownData)
			{
				intern->data = erealloc(intern->data, aiData.len);
				intern->len = aiData.len;
			}
			else if (allocData)
			{
				intern->data = emalloc(aiData.len);
				intern->ownData = 1;
				intern->len = aiData.len;
			}

			memcpy(intern->data, aiData.data, aiData.len);
		}
		else
		{
			if (intern->data && intern->ownData)
				efree(intern->data);

			intern->data = NULL;
			intern->len = 0;
			intern->ownData = 0;
		}

		SBFreeArrayZValInfo(&aiData);
	}
	else
	{
		SBErrorExpectsArguments("(array of byte/string/null)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TSBPointer, getLength)
{
	if (ZEND_NUM_ARGS() == 0)
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(getThis() TSRMLS_CC);
		RETURN_LONG((long)intern->len);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TSBPointer, setLength)
{
	long lSize;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &lSize) == SUCCESS)
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if (intern->data && intern->ownData)
			intern->data = erealloc(intern->data, lSize);
		else
			intern->data = emalloc(lSize);

		intern->ownData = 1;
		intern->len = lSize;
	}
	else
	{
		SBErrorExpectsArguments("(long)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TSBPointer, isNULL)
{
	if (ZEND_NUM_ARGS() == 0)
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(getThis() TSRMLS_CC);
		RETURN_BOOL(intern->data == NULL);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TSBPointer, setNULL)
{
	if (ZEND_NUM_ARGS() == 0)
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if (intern->data && intern->ownData)
			efree(intern->data);

		intern->data = NULL;
		intern->len = 0;
		intern->ownData = 0;
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TSBPointer, fromObject)
{
	zval *oObj;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!", &oObj, TObject_ce_ptr) == SUCCESS)
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if (intern->data && intern->ownData)
			efree(intern->data);

		intern->data = (void *)SBGetObjectHandle(oObj TSRMLS_CC);
		intern->len = 0;
		intern->ownData = 0;
	}
	else
	{
		SBErrorExpectsArguments("(TObject)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TSBPointer, toObject)
{
	if (ZEND_NUM_ARGS() == 0)
	{
		php_sbb_pointer * intern = (php_sbb_pointer *)zend_object_store_get_object(getThis() TSRMLS_CC);
		TElClassHandle handle = (TElClassHandle)intern->data;
		SBInitObject(return_value, TObject_ce_ptr, handle TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBPointer___construct, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBPointer_getData, 0, 0, 0)
	ZEND_ARG_INFO(0, MaxLength)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBPointer_setData, 0, 0, 1)
	ZEND_ARG_TYPE_INFO(0, Data, 0, 1)
	ZEND_ARG_INFO(0, AllocData)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBPointer_getLength, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBPointer_setLength, 0, 0, 1)
	ZEND_ARG_INFO(0, NewLength)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBPointer_isNULL, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBPointer_setNULL, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBPointer_fromObject, 0, 0, 1)
	ZEND_ARG_OBJ_INFO(0, Value, TObject, 1)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBPointer_toObject, 0, 0, 0)
ZEND_END_ARG_INFO()

static zend_function_entry TSBPointer_methods[] = {
	PHP_ME(TSBPointer, __construct, arginfo_TSBPointer___construct, ZEND_ACC_PUBLIC)
	PHP_ME(TSBPointer, __clone, NULL, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
	PHP_ME(TSBPointer, getData, arginfo_TSBPointer_getData, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(TSBPointer, setData, arginfo_TSBPointer_setData, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(TSBPointer, getLength, arginfo_TSBPointer_getLength, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(TSBPointer, setLength, arginfo_TSBPointer_setLength, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(TSBPointer, isNULL, arginfo_TSBPointer_isNULL, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(TSBPointer, setNULL, arginfo_TSBPointer_setNULL, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(TSBPointer, fromObject, arginfo_TSBPointer_fromObject, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_ME(TSBPointer, toObject, arginfo_TSBPointer_toObject, ZEND_ACC_PUBLIC | ZEND_ACC_FINAL)
	PHP_FE_END
};

void * SBGetStructPointer(zval *obj TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT))
		return NULL;
	else
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(obj TSRMLS_CC);
		return intern->data;
	}
}

void SBGetStructPointerValueTo(zval *obj, zend_class_entry *struct_ce, void * dst_data TSRMLS_DC)
{
	if ((Z_TYPE_P(obj) == IS_OBJECT) && (Z_OBJCE_P(obj) == struct_ce))
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(obj TSRMLS_CC);
		memcpy(dst_data, intern->data, intern->len);
	}
}

void SBSetStruct(zval *obj, void *data, size_t size TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT))
		return;
	else
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(obj TSRMLS_CC);

		if (intern->data && intern->ownData)
			efree(intern->data);

		intern->data = emalloc(size);
		intern->len = size;
		intern->ownData = 1;
		memcpy(intern->data, data, size);
	}
}

void SBSetStructPointer(zval *obj, void *data, size_t size TSRMLS_DC)
{
	if ((NULL == obj) || (Z_TYPE_P(obj) != IS_OBJECT))
		return;
	else
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(obj TSRMLS_CC);

		if (intern->data == data)
			return;

		/* don't free, as data could be used internally (e.g. SBMath.LSwap), will be cleared by Zend
		if (sbb_struct->data && sbb_struct->ownData)
		efree(sbb_struct->data);
		*/

		intern->data = data;
		intern->len = size;
		intern->ownData = 0;
	}
}

void SBInitStructObject(zval *obj, zend_class_entry *struct_ce, void * data, size_t size TSRMLS_DC)
{
	if (data != NULL)
	{
		php_sbb_struct * intern;
		object_init_ex(obj, struct_ce);
		intern = (php_sbb_struct *)zend_object_store_get_object(obj TSRMLS_CC);
		intern->data = data;
		intern->len = size;
		intern->ownData = 0;
	}
	else
	{
		INIT_ZVAL(*obj);
		ZVAL_NULL(obj);
	}
}

#pragma endregion

#pragma region TSBBaseStruct

static zend_object_handlers TSBBaseStruct_handlers;

static void PHP_TSBBaseStruct_Free(void *object TSRMLS_DC)
{
	php_sbb_struct *intern = (php_sbb_struct *)object;

	if (!intern)
		return;

	if (intern->data && intern->ownData)
		efree(intern->data);
	intern->data = NULL;
	intern->len = 0;
	intern->ownData = 0;

	zend_object_std_dtor(&intern->zo TSRMLS_CC);

	efree(intern);
}

static zend_object_value PHP_TSBBaseStruct_New(zend_class_entry *class_type TSRMLS_DC)
{
	zend_object_value retval;
	php_sbb_struct *intern;

	/* Allocate memory for it */
	intern = emalloc(sizeof(php_sbb_struct));
	memset(&intern->zo, 0, sizeof(php_sbb_struct));

	zend_object_std_init(&intern->zo, class_type TSRMLS_CC);
#if PHP_VERSION_ID >= 50400L
	object_properties_init(&intern->zo, class_type);
#endif

	retval.handle = zend_objects_store_put(intern, (zend_objects_store_dtor_t)zend_objects_destroy_object, (zend_objects_free_object_storage_t)PHP_TSBBaseStruct_Free, NULL TSRMLS_CC);
	retval.handlers = &TSBBaseStruct_handlers;

	return retval;
}

void SBStuctGetFieldInt8(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	int8_t res = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(int8_t)))
			res = ((int8_t *)intern->data)[field_offset];
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);

		RETURN_LONG((long)res);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

void SBStuctGetFieldUInt8(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	uint8_t res = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(uint8_t)))
			res = ((uint8_t *)intern->data)[field_offset];
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);

		RETURN_LONG((long)res);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

void SBStuctGetFieldInt16(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	int16_t res = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(int16_t)))
			res = *(int16_t *)((uint8_t *)intern->data + field_offset);
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);

		RETURN_LONG((long)res);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

void SBStuctGetFieldUInt16(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	uint16_t res = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(uint16_t)))
			res = *(uint16_t *)((uint8_t *)intern->data + field_offset);
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);

		RETURN_LONG((long)res);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

void SBStuctGetFieldInt32(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	int32_t res = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(int32_t)))
			res = *(int32_t *)((uint8_t *)intern->data + field_offset);
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);

		RETURN_LONG((long)res);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

void SBStuctGetFieldUInt32(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	uint32_t res = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(uint32_t)))
			res = *(uint32_t *)((uint8_t *)intern->data + field_offset);
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);

		RETURN_LONG((long)res);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

void SBStuctGetFieldInt64(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	int64_t res = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(int64_t)))
			res = *(int64_t *)((uint8_t *)intern->data + field_offset);
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);

		RETURN_LONG((long)res);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

void SBStuctGetFieldUInt64(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	uint64_t res = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(uint64_t)))
			res = *(uint64_t *)((uint8_t *)intern->data + field_offset);
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);

		RETURN_LONG((long)res);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

void SBStuctSetFieldInt8(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	long val = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(int8_t)))
			((int8_t *)intern->data)[field_offset] = (int8_t)val;
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

void SBStuctSetFieldUInt8(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	long val = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(uint8_t)))
			((uint8_t *)intern->data)[field_offset] = (uint8_t)val;
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

void SBStuctSetFieldInt16(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	long val = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(int16_t)))
			*(int16_t *)((uint8_t *)intern->data + field_offset) = (int16_t)val;
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

void SBStuctSetFieldUInt16(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	long val = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(uint16_t)))
			*(uint16_t *)((uint8_t *)intern->data + field_offset) = (uint16_t)val;
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

void SBStuctSetFieldInt32(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	long val = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(int32_t)))
			*(int32_t *)((uint8_t *)intern->data + field_offset) = (int32_t)val;
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

void SBStuctSetFieldUInt32(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	long val = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(uint32_t)))
			*(uint32_t *)((uint8_t *)intern->data + field_offset) = (uint32_t)val;
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

void SBStuctSetFieldInt64(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	long val = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(int64_t)))
			*(int64_t *)((uint8_t *)intern->data + field_offset) = (int64_t)val;
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

void SBStuctSetFieldUInt64(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	long val = 0;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &val) == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(uint64_t)))
			*(uint64_t *)((uint8_t *)intern->data + field_offset) = (uint64_t)val;
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

void SBStuctGetFieldStringA(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(void *)))
		{
			void *pStr = *(void **)((uint8_t *)intern->data + field_offset);
			char *pBuffer = NULL;
			int32_t szBuffer = 0;
			int32_t _err = SBGetVCLStringA(pStr, NULL, &szBuffer);
			if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
			{
				pBuffer = emalloc(szBuffer + 1);
				SBCheckError(SBGetVCLStringA(pStr, pBuffer, &szBuffer) TSRMLS_CC);
			}
			else if (_err != SB_ERROR_OK)
			{
				SBThrowException(_err TSRMLS_CC);
				RETURN_FALSE;
			}
			
			if (NULL == pBuffer) pBuffer = emalloc(1);
			pBuffer[szBuffer] = 0;
			RETURN_STRINGL(pBuffer, szBuffer, 0);
		}
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

void SBStuctSetFieldStringA(INTERNAL_FUNCTION_PARAMETERS, int field_offset)
{
	char *pValue;
	int szValue;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "s", &pValue, &szValue) == SUCCESS)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		if ((intern != NULL) && (intern->data != NULL) && (intern->len >= field_offset + sizeof(int64_t)))
		{
			void * pStr = *(void **)((uint8_t *)intern->data + field_offset);
			SBCheckError(SBSetVCLStringA(&pStr, pValue, (int32_t)szValue) TSRMLS_CC);
			*(void **)((uint8_t *)intern->data + field_offset) = pStr;
		}
		else
			SBThrowException(SB_ERROR_INVALID_BUFFER_SIZE TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

/*
SB_PHP_METHOD(TSBBaseStruct, __construct)
{
}
*/

SB_PHP_METHOD(TSBBaseStruct, __clone)
{
	/* Should never be executable */
	zend_throw_exception(NULL, "Cannot clone object using __clone()", 0 TSRMLS_CC);
}

SB_PHP_METHOD(TSBBaseStruct, __get)
{
	char *arg = NULL;
	int arg_len;
	if ((zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &arg, &arg_len) == FAILURE) || (arg_len < 1))
		return;
	else
	{
		zval *object = getThis();
		zval *result_ptr = NULL;
		char *name = emalloc(arg_len + 5);
		memcpy(name + 4, arg, arg_len);
		name[0] = 'g';
		name[1] = 'e';
		name[2] = 't';
		name[3] = '_';
		name[arg_len + 4] = 0;

		zend_call_method(&object, NULL, NULL, name, arg_len + 4, &result_ptr, 0, NULL, NULL TSRMLS_CC);

		efree(name);
		if (result_ptr) {
			RETVAL_ZVAL_FAST(result_ptr);
			zval_ptr_dtor(&result_ptr);
		}
	}
}

SB_PHP_METHOD(TSBBaseStruct, __set)
{
	zval *zValue = NULL;
	char *arg = NULL;
	int arg_len;
	if ((zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sz", &arg, &arg_len, &zValue) == FAILURE) || (arg_len < 1))
		return;
	else
	{
		zval *object = getThis();
		char *name = emalloc(arg_len + 5);
		memcpy(name + 4, arg, arg_len);
		name[0] = 's';
		name[1] = 'e';
		name[2] = 't';
		name[3] = '_';
		name[arg_len + 4] = 0;
		zend_call_method(&object, NULL, NULL, name, arg_len + 4, NULL, 1, zValue, NULL TSRMLS_CC);
		efree(name);
	}
}

SB_PHP_METHOD(TSBBaseStruct, __isset)
{
	zend_error(E_COMPILE_ERROR, "Cannot use isset()/empty() on the result of a function/property call (you can use \"null !== func()\" instead)");
}

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBBaseStruct___get, 0, 0, 0)
	ZEND_ARG_INFO(0, name)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBBaseStruct___set, 0, 0, 0)
	ZEND_ARG_INFO(0, name)
	ZEND_ARG_INFO(0, value)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TSBBaseStruct___isset, 0, 0, 0)
	ZEND_ARG_INFO(0, name)
ZEND_END_ARG_INFO()

static zend_function_entry TSBBaseStruct_methods[] = {
	// PHP_ME(TSBBaseStruct, __construct, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TSBBaseStruct, __clone, NULL, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
	PHP_ME(TSBBaseStruct, __get, arginfo_TSBBaseStruct___get, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
	PHP_ME(TSBBaseStruct, __set, arginfo_TSBBaseStruct___set, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
	PHP_ME(TSBBaseStruct, __isset, arginfo_TSBBaseStruct___isset, ZEND_ACC_PRIVATE | ZEND_ACC_FINAL)
	PHP_FE_END
};

#pragma endregion 

#pragma region TSBBaseEnum/TSBBaseFlags

zend_class_entry *TSBBaseEnum_ce_ptr;
zend_class_entry *TSBBaseFlags_ce_ptr;

zend_class_entry *TFileMode_ce_ptr;

#pragma endregion 

#pragma region DateTime

int64_t SBGetDateTime(zval *obj TSRMLS_DC)
{
	if ((obj == NULL) || (Z_TYPE_P(obj) != IS_OBJECT) || (Z_OBJCE_P(obj) != php_date_get_date_ce()))
		return 0;
	else
	{
		php_date_obj * intern = (php_date_obj *)zend_object_store_get_object(obj TSRMLS_CC);
		return (int64_t)intern->time->sse;
	}
}

void SBSetDateTime(zval *obj, int64_t dt TSRMLS_DC)
{
	if ((obj == NULL) || (Z_TYPE_P(obj) != IS_OBJECT) || (Z_OBJCE_P(obj) != php_date_get_date_ce()))
		return;
	else
	{
		php_date_obj * intern = (php_date_obj *)zend_object_store_get_object(obj TSRMLS_CC);
		/* internal methods
		if (NULL == intern->time)
		intern->time = timelib_time_ctor();

		timelib_unixtime2gmt(intern->time, dt);
		*/
		if (dt >= 0)
		{
			// the U option does not support negative timestamps (before 1970)
			zend_error_handling error_handling;
			char * time_str = emalloc(32);
			int time_str_len = slprintf(time_str, 32, "%lld", dt);

			zend_replace_error_handling(EH_THROW, NULL, &error_handling TSRMLS_CC);
			php_date_initialize(intern, time_str, time_str_len, "U", NULL, 1 TSRMLS_CC);
			zend_restore_error_handling(&error_handling TSRMLS_CC);

			efree(time_str);
		}
		else
		{
			#define DATE_INTERVAL "DateInterval"
			zend_error_handling error_handling;
			zend_class_entry **pce;

			zend_replace_error_handling(EH_THROW, NULL, &error_handling TSRMLS_CC);
			if (zend_lookup_class(DATE_INTERVAL, sizeof(DATE_INTERVAL) - 1, &pce TSRMLS_CC) == FAILURE)
				pce = NULL;

			php_date_initialize(intern, "0", 1, "U", NULL, 1 TSRMLS_CC);
			if (pce)
			{
				zval *IntervalObj, *arg1;
				char * time_str = emalloc(32);
				int time_str_len = slprintf(time_str, 32, "PT%lldS", (-dt));

				MAKE_STD_ZVAL(IntervalObj);
				MAKE_STD_ZVAL(arg1);
				
				object_init_ex(IntervalObj, *pce);
				ZVAL_STRINGL(arg1, time_str, time_str_len, 0);
				
				zend_call_method_with_1_params(&IntervalObj, *pce, NULL, ZEND_CONSTRUCTOR_FUNC_NAME, NULL, arg1);
				zend_call_method_with_1_params(&obj, php_date_get_date_ce(), NULL, "sub", NULL, IntervalObj);

				zval_ptr_dtor(&IntervalObj);
				zval_ptr_dtor(&arg1);
			}
			else
				zend_error(E_ERROR, "Failed to lookup class: " DATE_INTERVAL);

			zend_restore_error_handling(&error_handling TSRMLS_CC);
		}
	}
}

#pragma endregion

#pragma region TSBInt64

#pragma endregion 

void * SBSetEventData(zval * func /*, zend_class_entry * intf_ce */ TSRMLS_DC)
{
	zval *res;
	if (Z_TYPE_P(func) == IS_NULL)
		return NULL;
	else if ((Z_TYPE_P(func) == IS_STRING) || (Z_TYPE_P(func) == IS_ARRAY) || (Z_TYPE_P(func) == IS_CALLABLE))
	{
		char *callback_name;
		if (!zend_is_callable(func, 0, &callback_name TSRMLS_CC))
		{
			zend_error(E_WARNING, "%s() expects the argument (%s) to be a valid callback",
				get_active_function_name(TSRMLS_C), callback_name ? callback_name : "unknown");
			efree(callback_name);
			return NULL;
		}

		efree(callback_name);

	}
	/*
	else if ((Z_TYPE_P(func) == IS_OBJECT) && is_derived_class(Z_OBJCE_P(func), intf_ce))  // intf_ce is abstract class, so not comparing with it
	{
		// check if function not abstract
	}
	*/
	else  
	{
		zend_error(E_WARNING, "%s() expects the argument to be a valid callback",
			get_active_function_name(TSRMLS_C));
		return NULL;
	}

	ALLOC_ZVAL(res);
	INIT_PZVAL_COPY(res, func);
	zval_copy_ctor(res);
	return (void *)res;
}

void SB_CALLBACK SBEventFreeData(void * data)
{
	if (data)
	{
		zval * func = (zval *)data;
		zval_ptr_dtor(&func);
	}
}

int SBCallEvent(zval * func, zval **retval_ptr_ptr, zend_uint param_count, zval **params[] TSRMLS_DC)
{
	if (func == NULL)
		return FAILURE;

	if ((Z_TYPE_P(func) == IS_STRING) || (Z_TYPE_P(func) == IS_ARRAY) || (Z_TYPE_P(func) == IS_CALLABLE))
	{
		zend_fcall_info fci;
		int res;

		fci.size = sizeof(fci);
		fci.function_table = EG(function_table);
		fci.function_name = func;
		fci.symbol_table = NULL;
		fci.object_ptr = NULL;
		fci.retval_ptr_ptr = retval_ptr_ptr;

		fci.param_count = param_count;
		fci.params = params;
		fci.no_separation = 1;

		res = zend_call_function(&fci, NULL TSRMLS_CC);
		if (res == FAILURE)
		{
			zval **method;
			zval **obj;

			if (Z_TYPE_P(func) == IS_STRING) {
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to call handler %s()", Z_STRVAL_P(func));
			}
			else if (zend_hash_index_find(Z_ARRVAL_P(func), 0, (void **)&obj) == SUCCESS &&
				zend_hash_index_find(Z_ARRVAL_P(func), 1, (void **)&method) == SUCCESS &&
				Z_TYPE_PP(obj) == IS_OBJECT &&
				Z_TYPE_PP(method) == IS_STRING) {
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to call handler %s::%s()", Z_OBJCE_PP(obj)->name, Z_STRVAL_PP(method));
			}
			else
				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unable to call handler");
		}

		return res;
	}
	else
		return FAILURE;
}

void Register_Core(TSRMLS_D)
{
	zend_class_entry ce;
	SB_OUTPUT_DEBUG("Register Core classes")
	INIT_CLASS_ENTRY(ce, "SBException", SBException_methods);
	SBException_ce_ptr = zend_register_internal_class_ex(&ce, zend_exception_get_default(TSRMLS_C), NULL TSRMLS_CC);

	zend_declare_property_long(SBException_ce_ptr, "errorcode", sizeof("errorcode") - 1, 0, ZEND_ACC_PROTECTED TSRMLS_CC);
	zend_declare_property_long(SBException_ce_ptr, "supplerrorcode", sizeof("supplerrorcode") - 1, 0, ZEND_ACC_PROTECTED TSRMLS_CC);
	zend_declare_property_string(SBException_ce_ptr, "errorname", sizeof("errorname") - 1, "", ZEND_ACC_PROTECTED TSRMLS_CC);
	zend_declare_property_string(SBException_ce_ptr, "errormessage", sizeof("errormessage") - 1, "", ZEND_ACC_PROTECTED TSRMLS_CC);
	zend_declare_property_string(SBException_ce_ptr, "errorstacktrace", sizeof("errorstacktrace") - 1, "", ZEND_ACC_PROTECTED TSRMLS_CC);	

	INIT_CLASS_ENTRY(ce, "SBInsufficientBufferSizeException", NULL);
	SBInsufficientBufferSizeException_ce_ptr = zend_register_internal_class_ex(&ce, zend_exception_get_default(TSRMLS_C), NULL TSRMLS_CC);

	INIT_CLASS_ENTRY(ce, "SBInvalidBufferSizeException", NULL);
	SBInvalidBufferSizeException_ce_ptr = zend_register_internal_class_ex(&ce, zend_exception_get_default(TSRMLS_C), NULL TSRMLS_CC);

	INIT_CLASS_ENTRY(ce, "SBInvalidParameterException", NULL);
	SBInvalidParameterException_ce_ptr = zend_register_internal_class_ex(&ce, zend_exception_get_default(TSRMLS_C), NULL TSRMLS_CC);

	INIT_CLASS_ENTRY(ce, "SBInternalException", NULL);
	SBInternalException_ce_ptr = zend_register_internal_class_ex(&ce, zend_exception_get_default(TSRMLS_C), NULL TSRMLS_CC);

	INIT_CLASS_ENTRY(ce, "TSBPointer", TSBPointer_methods);
	TSBPointer_ce_ptr = zend_register_internal_class(&ce TSRMLS_CC);
	TSBPointer_ce_ptr->create_object = PHP_TSBPointer_New;
	memcpy(&TSBPointer_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));
	TSBPointer_handlers.clone_obj = NULL;

	INIT_CLASS_ENTRY(ce, "TSBBaseStruct", TSBBaseStruct_methods);
	TSBBaseStruct_ce_ptr = zend_register_internal_class(&ce TSRMLS_CC);
	TSBBaseStruct_ce_ptr->create_object = PHP_TSBBaseStruct_New;
	memcpy(&TSBBaseStruct_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));
	TSBBaseStruct_handlers.clone_obj = NULL;

	INIT_CLASS_ENTRY(ce, "TSBBaseEnum", NULL);
	TSBBaseEnum_ce_ptr = zend_register_internal_class(&ce TSRMLS_CC);
	TSBBaseEnum_ce_ptr->ce_flags |= ZEND_ACC_EXPLICIT_ABSTRACT_CLASS;

	INIT_CLASS_ENTRY(ce, "TSBBaseFlags", NULL);
	TSBBaseFlags_ce_ptr = zend_register_internal_class(&ce TSRMLS_CC);
	TSBBaseFlags_ce_ptr->ce_flags |= ZEND_ACC_EXPLICIT_ABSTRACT_CLASS;

	INIT_CLASS_ENTRY(ce, "TFileMode", NULL);
	TFileMode_ce_ptr = zend_register_internal_class_ex(&ce, TSBBaseEnum_ce_ptr, NULL TSRMLS_CC);
	TFileMode_ce_ptr->ce_flags |= ZEND_ACC_FINAL_CLASS;
	SB_DECLARE_CLASS_LONG_CONST(TFileMode_ce_ptr, "fmOpenRead", 0)
	SB_DECLARE_CLASS_LONG_CONST(TFileMode_ce_ptr, "fmOpenWrite", 1)
	SB_DECLARE_CLASS_LONG_CONST(TFileMode_ce_ptr, "fmOpenReadWrite", 2)
	SB_DECLARE_CLASS_LONG_CONST(TFileMode_ce_ptr, "fmShareDenyNone", 0x0040)
	SB_DECLARE_CLASS_LONG_CONST(TFileMode_ce_ptr, "fmShareDenyRead", 0x0030)
	SB_DECLARE_CLASS_LONG_CONST(TFileMode_ce_ptr, "fmShareDenyWrite", 0x0020)
	SB_DECLARE_CLASS_LONG_CONST(TFileMode_ce_ptr, "fmShareExclusive", 0x0010)
#if defined(LINUX) || defined(MACOS) || defined(SB_IOS) || defined(CPU64) || defined(WINCE)
	// build with FPC
	SB_DECLARE_CLASS_LONG_CONST(TFileMode_ce_ptr, "fmCreate", 0xff00)
#else
	// build with Delphi
	SB_DECLARE_CLASS_LONG_CONST(TFileMode_ce_ptr, "fmCreate", 0xffff)
#endif
}
