#include "sbsrp.h"

SB_PHP_METHOD(TSRPContext, __construct)
{
	if (ZEND_NUM_ARGS() == 0)
	{
		php_sbb_struct * intern = (php_sbb_struct *)zend_object_store_get_object(getThis() TSRMLS_CC);
		intern->data = emalloc(sizeof(TSRPContext));
		intern->len = sizeof(TSRPContext);
		intern->ownData = 1;
	}
	else
		ZEND_WRONG_PARAM_COUNT();
}

SB_PHP_METHOD(TSRPContext, getSizeOf)
{
	if (ZEND_NUM_ARGS() > 0)
		ZEND_WRONG_PARAM_COUNT();

	RETURN_LONG(sizeof(TSRPContext));
}

zend_class_entry *TSRPContext_ce_ptr = NULL;

static zend_function_entry TSRPContext_methods[] = {
	PHP_ME(TSRPContext, __construct, NULL, ZEND_ACC_PUBLIC)
	PHP_ME(TSRPContext, getSizeOf, NULL, ZEND_ACC_PUBLIC | ZEND_ACC_STATIC)
	PHP_FE_END
};

void Register_TSRPContext(TSRMLS_D)
{
	zend_class_entry ce;
	if (TSRPContext_ce_ptr != NULL)
		return;
	INIT_CLASS_ENTRY(ce, "TSRPContext", TSRPContext_methods);
	TSRPContext_ce_ptr = zend_register_internal_class_ex(&ce, TSBBaseStruct_ce_ptr, NULL TSRMLS_CC);
}

zend_class_entry *TSSRPPrimeLen_ce_ptr = NULL;

zend_class_entry *TSBSRPCredentialPrimeLen_ce_ptr = NULL;

zend_class_entry *TElSRPCredential_ce_ptr = NULL;

SB_PHP_METHOD(TElSRPCredential, Generate)
{
	char *sPassword;
	char *sUsername;
	int sPassword_len;
	int sUsername_len;
	long fPrimeLen;
	SBArrayZValInfo aiGenerator;
	SBArrayZValInfo aiPrime;
	zval *zaGenerator;
	zval *zaPrime;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "ssl", &sUsername, &sUsername_len, &sPassword, &sPassword_len, &fPrimeLen) == SUCCESS)
	{
		SBCheckError(TElSRPCredential_Generate(SBGetObjectHandle(getThis() TSRMLS_CC), sUsername, (int32_t)sUsername_len, sPassword, (int32_t)sPassword_len, (TSBSRPCredentialPrimeLenRaw)fPrimeLen) TSRMLS_CC);
	}
	else if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "sszz", &sUsername, &sUsername_len, &sPassword, &sPassword_len, &zaPrime, &zaGenerator) == SUCCESS) && ((IS_STRING == Z_TYPE_P(zaPrime)) || (IS_ARRAY == Z_TYPE_P(zaPrime)) || (IS_NULL == Z_TYPE_P(zaPrime))) && ((IS_STRING == Z_TYPE_P(zaGenerator)) || (IS_ARRAY == Z_TYPE_P(zaGenerator)) || (IS_NULL == Z_TYPE_P(zaGenerator))))
	{
		if (!SBGetByteArrayFromZVal(zaPrime, &aiPrime TSRMLS_CC)) RETURN_FALSE;
		if (!SBGetByteArrayFromZVal(zaGenerator, &aiGenerator TSRMLS_CC)) RETURN_FALSE;
		SBCheckError(TElSRPCredential_Generate_1(SBGetObjectHandle(getThis() TSRMLS_CC), sUsername, (int32_t)sUsername_len, sPassword, (int32_t)sPassword_len, aiPrime.data, aiPrime.len, aiGenerator.data, aiGenerator.len) TSRMLS_CC);
		SBFreeArrayZValInfo(&aiPrime);
		SBFreeArrayZValInfo(&aiGenerator);
	}
	else
	{
		SBErrorExpectsArguments("(string, string, integer) or (string, string, array of byte|string|NULL, array of byte|string|NULL)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredential, Load)
{
	char *sData;
	char *sUsername;
	int sData_len;
	int sUsername_len;
	SBArrayZValInfo aiGenerator;
	SBArrayZValInfo aiPrime;
	SBArrayZValInfo aiSalt;
	SBArrayZValInfo aiVerifier;
	zval *zaGenerator;
	zval *zaPrime;
	zval *zaSalt;
	zval *zaVerifier;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "s", &sData, &sData_len) == SUCCESS)
	{
		SBCheckError(TElSRPCredential_Load(SBGetObjectHandle(getThis() TSRMLS_CC), sData, (int32_t)sData_len) TSRMLS_CC);
	}
	else if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "szzzz", &sUsername, &sUsername_len, &zaSalt, &zaPrime, &zaGenerator, &zaVerifier) == SUCCESS) && ((IS_STRING == Z_TYPE_P(zaSalt)) || (IS_ARRAY == Z_TYPE_P(zaSalt)) || (IS_NULL == Z_TYPE_P(zaSalt))) && ((IS_STRING == Z_TYPE_P(zaPrime)) || (IS_ARRAY == Z_TYPE_P(zaPrime)) || (IS_NULL == Z_TYPE_P(zaPrime))) && ((IS_STRING == Z_TYPE_P(zaGenerator)) || (IS_ARRAY == Z_TYPE_P(zaGenerator)) || (IS_NULL == Z_TYPE_P(zaGenerator))) && ((IS_STRING == Z_TYPE_P(zaVerifier)) || (IS_ARRAY == Z_TYPE_P(zaVerifier)) || (IS_NULL == Z_TYPE_P(zaVerifier))))
	{
		if (!SBGetByteArrayFromZVal(zaSalt, &aiSalt TSRMLS_CC)) RETURN_FALSE;
		if (!SBGetByteArrayFromZVal(zaPrime, &aiPrime TSRMLS_CC)) RETURN_FALSE;
		if (!SBGetByteArrayFromZVal(zaGenerator, &aiGenerator TSRMLS_CC)) RETURN_FALSE;
		if (!SBGetByteArrayFromZVal(zaVerifier, &aiVerifier TSRMLS_CC)) RETURN_FALSE;
		SBCheckError(TElSRPCredential_Load_1(SBGetObjectHandle(getThis() TSRMLS_CC), sUsername, (int32_t)sUsername_len, aiSalt.data, aiSalt.len, aiPrime.data, aiPrime.len, aiGenerator.data, aiGenerator.len, aiVerifier.data, aiVerifier.len) TSRMLS_CC);
		SBFreeArrayZValInfo(&aiSalt);
		SBFreeArrayZValInfo(&aiPrime);
		SBFreeArrayZValInfo(&aiGenerator);
		SBFreeArrayZValInfo(&aiVerifier);
	}
	else
	{
		SBErrorExpectsArguments("(string) or (string, array of byte|string|NULL, array of byte|string|NULL, array of byte|string|NULL, array of byte|string|NULL)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredential, Save)
{
	uint32_t _err;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		char *sOutResult = NULL;
		int32_t sOutResult_len = 0;
		_err = TElSRPCredential_Save(SBGetObjectHandle(getThis() TSRMLS_CC), sOutResult, &sOutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			sOutResult = emalloc(sOutResult_len + 1);
			SBCheckError(SBGetLastReturnStringA(-771191387, 1, sOutResult, &sOutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		if (NULL == sOutResult) sOutResult = emalloc(1);
		sOutResult[sOutResult_len] = 0;
		RETURN_STRINGL(sOutResult, sOutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredential, get_Username)
{
	uint32_t _err;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		char *sOutResult = NULL;
		int32_t sOutResult_len = 0;
		_err = TElSRPCredential_get_Username(SBGetObjectHandle(getThis() TSRMLS_CC), sOutResult, &sOutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			sOutResult = emalloc(sOutResult_len + 1);
			SBCheckError(SBGetLastReturnStringA(-1256693578, 1, sOutResult, &sOutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		if (NULL == sOutResult) sOutResult = emalloc(1);
		sOutResult[sOutResult_len] = 0;
		RETURN_STRINGL(sOutResult, sOutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredential, get_Prime)
{
	uint32_t _err;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		uint8_t *u8OutResult = NULL;
		int32_t u8OutResult_len = 0;
		_err = TElSRPCredential_get_Prime(SBGetObjectHandle(getThis() TSRMLS_CC), u8OutResult, &u8OutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			u8OutResult = emalloc(u8OutResult_len + 1);
			SBCheckError(SBGetLastReturnBuffer(-228639922, 1, u8OutResult, &u8OutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		if (NULL == u8OutResult) u8OutResult = emalloc(1);
		u8OutResult[u8OutResult_len] = 0;
		RETURN_STRINGL(u8OutResult, u8OutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredential, get_Generator)
{
	uint32_t _err;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		uint8_t *u8OutResult = NULL;
		int32_t u8OutResult_len = 0;
		_err = TElSRPCredential_get_Generator(SBGetObjectHandle(getThis() TSRMLS_CC), u8OutResult, &u8OutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			u8OutResult = emalloc(u8OutResult_len + 1);
			SBCheckError(SBGetLastReturnBuffer(-192798531, 1, u8OutResult, &u8OutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		if (NULL == u8OutResult) u8OutResult = emalloc(1);
		u8OutResult[u8OutResult_len] = 0;
		RETURN_STRINGL(u8OutResult, u8OutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredential, set_Generator)
{
	SBArrayZValInfo aiValue;
	zval *zaValue;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "z", &zaValue) == SUCCESS) && ((IS_STRING == Z_TYPE_P(zaValue)) || (IS_ARRAY == Z_TYPE_P(zaValue)) || (IS_NULL == Z_TYPE_P(zaValue))))
	{
		if (!SBGetByteArrayFromZVal(zaValue, &aiValue TSRMLS_CC)) RETURN_FALSE;
		SBCheckError(TElSRPCredential_set_Generator(SBGetObjectHandle(getThis() TSRMLS_CC), aiValue.data, aiValue.len) TSRMLS_CC);
		SBFreeArrayZValInfo(&aiValue);
	}
	else
	{
		SBErrorExpectsArguments("(array of byte|string|NULL)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredential, get_Salt)
{
	uint32_t _err;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		uint8_t *u8OutResult = NULL;
		int32_t u8OutResult_len = 0;
		_err = TElSRPCredential_get_Salt(SBGetObjectHandle(getThis() TSRMLS_CC), u8OutResult, &u8OutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			u8OutResult = emalloc(u8OutResult_len + 1);
			SBCheckError(SBGetLastReturnBuffer(687009396, 1, u8OutResult, &u8OutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		if (NULL == u8OutResult) u8OutResult = emalloc(1);
		u8OutResult[u8OutResult_len] = 0;
		RETURN_STRINGL(u8OutResult, u8OutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredential, get_Verifier)
{
	uint32_t _err;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		uint8_t *u8OutResult = NULL;
		int32_t u8OutResult_len = 0;
		_err = TElSRPCredential_get_Verifier(SBGetObjectHandle(getThis() TSRMLS_CC), u8OutResult, &u8OutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			u8OutResult = emalloc(u8OutResult_len + 1);
			SBCheckError(SBGetLastReturnBuffer(-1227412343, 1, u8OutResult, &u8OutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		if (NULL == u8OutResult) u8OutResult = emalloc(1);
		u8OutResult[u8OutResult_len] = 0;
		RETURN_STRINGL(u8OutResult, u8OutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredential, __construct)
{
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		TElClassHandle hoOutResult = NULL;
		SBCheckError(TElSRPCredential_Create(&hoOutResult) TSRMLS_CC);
		SBSetObjectHandle(getThis(), hoOutResult, ohTrue TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential_Generate, 0, 0, 3)
	ZEND_ARG_INFO(0, Username)
	ZEND_ARG_INFO(0, Password)
	ZEND_ARG_TYPE_INFO(0, PrimeLen_or_Prime, 0, 1)
	ZEND_ARG_TYPE_INFO(0, Generator, 0, 1)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential_Load, 0, 0, 1)
	ZEND_ARG_INFO(0, Data_or_Username)
	ZEND_ARG_TYPE_INFO(0, Salt, 0, 1)
	ZEND_ARG_TYPE_INFO(0, Prime, 0, 1)
	ZEND_ARG_TYPE_INFO(0, Generator, 0, 1)
	ZEND_ARG_TYPE_INFO(0, Verifier, 0, 1)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential_Save, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential_get_Username, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential_get_Prime, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential_get_Generator, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential_set_Generator, 0, 0, 1)
	ZEND_ARG_TYPE_INFO(0, Value, 0, 1)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential_get_Salt, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential_get_Verifier, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredential___construct, 0, 0, 0)
ZEND_END_ARG_INFO()

static zend_function_entry TElSRPCredential_methods[] = {
	PHP_ME(TElSRPCredential, Generate, arginfo_TElSRPCredential_Generate, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredential, Load, arginfo_TElSRPCredential_Load, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredential, Save, arginfo_TElSRPCredential_Save, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredential, get_Username, arginfo_TElSRPCredential_get_Username, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredential, get_Prime, arginfo_TElSRPCredential_get_Prime, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredential, get_Generator, arginfo_TElSRPCredential_get_Generator, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredential, set_Generator, arginfo_TElSRPCredential_set_Generator, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredential, get_Salt, arginfo_TElSRPCredential_get_Salt, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredential, get_Verifier, arginfo_TElSRPCredential_get_Verifier, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredential, __construct, arginfo_TElSRPCredential___construct, ZEND_ACC_CTOR | ZEND_ACC_PUBLIC)
	PHP_FE_END
};

void Register_TElSRPCredential(TSRMLS_D)
{
	zend_class_entry ce;
	if (TElSRPCredential_ce_ptr != NULL)
		return;

	INIT_CLASS_ENTRY(ce, "TElSRPCredential", TElSRPCredential_methods);
	if (NULL == TObject_ce_ptr)
		Register_TObject(TSRMLS_C);
	TElSRPCredential_ce_ptr = zend_register_internal_class_ex(&ce, TObject_ce_ptr, NULL TSRMLS_CC);
}

zend_class_entry *TElSRPCredentialStore_ce_ptr = NULL;

SB_PHP_METHOD(TElSRPCredentialStore, Add)
{
	char *sPassword;
	char *sUsername;
	int sPassword_len;
	int sUsername_len;
	long fPrimeLen;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		int32_t l4OutResultRaw = 0;
		SBCheckError(TElSRPCredentialStore_Add(SBGetObjectHandle(getThis() TSRMLS_CC), &l4OutResultRaw) TSRMLS_CC);
		RETURN_LONG((long)l4OutResultRaw);
	}
	else if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "ssl", &sUsername, &sUsername_len, &sPassword, &sPassword_len, &fPrimeLen) == SUCCESS)
	{
		int32_t l4OutResultRaw = 0;
		SBCheckError(TElSRPCredentialStore_Add_1(SBGetObjectHandle(getThis() TSRMLS_CC), sUsername, (int32_t)sUsername_len, sPassword, (int32_t)sPassword_len, (TSBSRPCredentialPrimeLenRaw)fPrimeLen, &l4OutResultRaw) TSRMLS_CC);
		RETURN_LONG((long)l4OutResultRaw);
	}
	else
	{
		SBErrorExpectsArguments("() or (string, string, integer)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredentialStore, Remove)
{
	long l4Index;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &l4Index) == SUCCESS)
	{
		SBCheckError(TElSRPCredentialStore_Remove(SBGetObjectHandle(getThis() TSRMLS_CC), (int32_t)l4Index) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredentialStore, Clear)
{
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		SBCheckError(TElSRPCredentialStore_Clear(SBGetObjectHandle(getThis() TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredentialStore, LoadFromStream)
{
	zval *oStream;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!", &oStream, TStream_ce_ptr) == SUCCESS)
	{
		SBCheckError(TElSRPCredentialStore_LoadFromStream(SBGetObjectHandle(getThis() TSRMLS_CC), SBGetObjectHandle(oStream TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(TStream)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredentialStore, SaveToStream)
{
	zval *oStream;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!", &oStream, TStream_ce_ptr) == SUCCESS)
	{
		SBCheckError(TElSRPCredentialStore_SaveToStream(SBGetObjectHandle(getThis() TSRMLS_CC), SBGetObjectHandle(oStream TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(TStream)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredentialStore, get_Credentials)
{
	long l4Index;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "l", &l4Index) == SUCCESS)
	{
		TElClassHandle hoOutResult = NULL;
		SBCheckError(TElSRPCredentialStore_get_Credentials(SBGetObjectHandle(getThis() TSRMLS_CC), (int32_t)l4Index, &hoOutResult) TSRMLS_CC);
		SBInitObject(return_value, TElSRPCredential_ce_ptr, hoOutResult TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(integer)" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredentialStore, get_Count)
{
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "") == SUCCESS)
	{
		int32_t l4OutResultRaw = 0;
		SBCheckError(TElSRPCredentialStore_get_Count(SBGetObjectHandle(getThis() TSRMLS_CC), &l4OutResultRaw) TSRMLS_CC);
		RETURN_LONG((long)l4OutResultRaw);
	}
	else
	{
		SBErrorExpectsArguments("()" TSRMLS_CC);
	}
}

SB_PHP_METHOD(TElSRPCredentialStore, __construct)
{
	zval *oAOwner;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!", &oAOwner, TComponent_ce_ptr) == SUCCESS)
	{
		TElClassHandle hoOutResult = NULL;
		SBCheckError(TElSRPCredentialStore_Create(SBGetObjectHandle(oAOwner TSRMLS_CC), &hoOutResult) TSRMLS_CC);
		SBSetObjectHandle(getThis(), hoOutResult, ohTrue TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(TComponent)" TSRMLS_CC);
	}
}

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredentialStore_Add, 0, 0, 0)
	ZEND_ARG_INFO(0, Username)
	ZEND_ARG_INFO(0, Password)
	ZEND_ARG_INFO(0, PrimeLen)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredentialStore_Remove, 0, 0, 1)
	ZEND_ARG_INFO(0, Index)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredentialStore_Clear, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredentialStore_LoadFromStream, 0, 0, 1)
	ZEND_ARG_OBJ_INFO(0, Stream, TStream, 1)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredentialStore_SaveToStream, 0, 0, 1)
	ZEND_ARG_OBJ_INFO(0, Stream, TStream, 1)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredentialStore_get_Credentials, 0, 0, 1)
	ZEND_ARG_INFO(0, Index)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredentialStore_get_Count, 0, 0, 0)
ZEND_END_ARG_INFO()

ZEND_BEGIN_ARG_INFO_EX(arginfo_TElSRPCredentialStore___construct, 0, 0, 1)
	ZEND_ARG_OBJ_INFO(0, AOwner, TComponent, 1)
ZEND_END_ARG_INFO()

static zend_function_entry TElSRPCredentialStore_methods[] = {
	PHP_ME(TElSRPCredentialStore, Add, arginfo_TElSRPCredentialStore_Add, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredentialStore, Remove, arginfo_TElSRPCredentialStore_Remove, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredentialStore, Clear, arginfo_TElSRPCredentialStore_Clear, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredentialStore, LoadFromStream, arginfo_TElSRPCredentialStore_LoadFromStream, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredentialStore, SaveToStream, arginfo_TElSRPCredentialStore_SaveToStream, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredentialStore, get_Credentials, arginfo_TElSRPCredentialStore_get_Credentials, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredentialStore, get_Count, arginfo_TElSRPCredentialStore_get_Count, ZEND_ACC_PUBLIC)
	PHP_ME(TElSRPCredentialStore, __construct, arginfo_TElSRPCredentialStore___construct, ZEND_ACC_CTOR | ZEND_ACC_PUBLIC)
	PHP_FE_END
};

void Register_TElSRPCredentialStore(TSRMLS_D)
{
	zend_class_entry ce;
	if (TElSRPCredentialStore_ce_ptr != NULL)
		return;

	INIT_CLASS_ENTRY(ce, "TElSRPCredentialStore", TElSRPCredentialStore_methods);
	if (NULL == TComponent_ce_ptr)
		Register_TComponent(TSRMLS_C);
	TElSRPCredentialStore_ce_ptr = zend_register_internal_class_ex(&ce, TComponent_ce_ptr, NULL TSRMLS_CC);
}

SB_PHP_FUNCTION(SBSRP, SrpInitContext)
{
	zval *oSRP;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oSRP))
	{
		SBCheckError(SBSRP_SrpInitContext((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(&TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpDestroyContext)
{
	zval *oSRP;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oSRP))
	{
		SBCheckError(SBSRP_SrpDestroyContext((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(&TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpServerInit)
{
	zval *oSRP;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oSRP))
	{
		SBCheckError(SBSRP_SrpServerInit((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(&TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpGetU)
{
	char *sProto;
	int sProto_len;
	TLInt * proU;
	zval *oA;
	zval *oB;
	zval *oN;
	zval *oU;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!O!O!sO", &oN, TLInt_ce_ptr, &oA, TLInt_ce_ptr, &oB, TLInt_ce_ptr, &sProto, &sProto_len, &oU, TLInt_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oU))
	{
		proU = (TLInt *)SBGetStructPointer(oU TSRMLS_CC);
		SBCheckError(SBSRP_SrpGetU((TLInt *)SBGetStructPointer(oN TSRMLS_CC), (TLInt *)SBGetStructPointer(oA TSRMLS_CC), (TLInt *)SBGetStructPointer(oB TSRMLS_CC), sProto, (int32_t)sProto_len, &proU) TSRMLS_CC);
		SBSetStructPointer(oU, proU, sizeof(TLInt) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(TLInt, TLInt, TLInt, string, &TLInt)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpGetUTLS)
{
	TLInt * proU;
	zval *oA;
	zval *oB;
	zval *oN;
	zval *oU;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!O!O!O", &oA, TLInt_ce_ptr, &oB, TLInt_ce_ptr, &oN, TLInt_ce_ptr, &oU, TLInt_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oU))
	{
		proU = (TLInt *)SBGetStructPointer(oU TSRMLS_CC);
		SBCheckError(SBSRP_SrpGetUTLS((TLInt *)SBGetStructPointer(oA TSRMLS_CC), (TLInt *)SBGetStructPointer(oB TSRMLS_CC), (TLInt *)SBGetStructPointer(oN TSRMLS_CC), &proU) TSRMLS_CC);
		SBSetStructPointer(oU, proU, sizeof(TLInt) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(TLInt, TLInt, TLInt, &TLInt)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpGetA)
{
	zval *oSRP;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS)
	{
		SBCheckError(SBSRP_SrpGetA((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpGetB)
{
	zval *oSRP;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS)
	{
		SBCheckError(SBSRP_SrpGetB((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpGetK)
{
	zval *oSRP;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS)
	{
		SBCheckError(SBSRP_SrpGetK((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpPrepareClientX)
{
	char *sPassword;
	char *sUsername;
	int sPassword_len;
	int sUsername_len;
	SBArrayZValInfo aiSalt;
	TLInt * proClX;
	zval *oClX;
	zval *zaSalt;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "sszO", &sUsername, &sUsername_len, &sPassword, &sPassword_len, &zaSalt, &oClX, TLInt_ce_ptr) == SUCCESS) && ((IS_STRING == Z_TYPE_P(zaSalt)) || (IS_ARRAY == Z_TYPE_P(zaSalt)) || (IS_NULL == Z_TYPE_P(zaSalt))) && PZVAL_IS_REF(oClX))
	{
		if (!SBGetByteArrayFromZVal(zaSalt, &aiSalt TSRMLS_CC)) RETURN_FALSE;
		proClX = (TLInt *)SBGetStructPointer(oClX TSRMLS_CC);
		SBCheckError(SBSRP_SrpPrepareClientX(sUsername, (int32_t)sUsername_len, sPassword, (int32_t)sPassword_len, aiSalt.data, aiSalt.len, &proClX) TSRMLS_CC);
		SBFreeArrayZValInfo(&aiSalt);
		SBSetStructPointer(oClX, proClX, sizeof(TLInt) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(string, string, array of byte|string|NULL, &TLInt)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpGetClientKey)
{
	zval *oSRP;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oSRP))
	{
		SBCheckError(SBSRP_SrpGetClientKey((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(&TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpPrepareClientKeyTLS)
{
	zval *oSRP;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oSRP))
	{
		SBCheckError(SBSRP_SrpPrepareClientKeyTLS((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(&TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpPrepareServerPreKeyTLS)
{
	zval *oSRP;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oSRP))
	{
		SBCheckError(SBSRP_SrpPrepareServerPreKeyTLS((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(&TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpPrepareServerKeyTLS)
{
	zval *oSRP;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oSRP))
	{
		SBCheckError(SBSRP_SrpPrepareServerKeyTLS((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(&TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpGetServerKey)
{
	zval *oSRP;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O", &oSRP, TSRPContext_ce_ptr) == SUCCESS) && PZVAL_IS_REF(oSRP))
	{
		SBCheckError(SBSRP_SrpGetServerKey((TSRPContext *)SBGetStructPointer(oSRP TSRMLS_CC)) TSRMLS_CC);
	}
	else
	{
		SBErrorExpectsArguments("(&TSRPContext)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpGetNewServerData)
{
	char *sUserName;
	char *sUserPassword;
	int sUserName_len;
	int sUserPassword_len;
	long fPrimeLen;
	SBArrayZValInfo aiG;
	SBArrayZValInfo aiN;
	SBArrayZValInfo aiSalt;
	SBArrayZValInfo aiV;
	uint32_t _err;
	zval *zaG;
	zval *zaN;
	zval *zaSalt;
	zval *zaV;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "sslzzzz", &sUserName, &sUserName_len, &sUserPassword, &sUserPassword_len, &fPrimeLen, &zaN, &zaG, &zaSalt, &zaV) == SUCCESS) && PZVAL_IS_REF(zaN) && ((IS_STRING == Z_TYPE_P(zaN)) || (IS_ARRAY == Z_TYPE_P(zaN))) && PZVAL_IS_REF(zaG) && ((IS_STRING == Z_TYPE_P(zaG)) || (IS_ARRAY == Z_TYPE_P(zaG))) && PZVAL_IS_REF(zaSalt) && ((IS_STRING == Z_TYPE_P(zaSalt)) || (IS_ARRAY == Z_TYPE_P(zaSalt))) && PZVAL_IS_REF(zaV) && ((IS_STRING == Z_TYPE_P(zaV)) || (IS_ARRAY == Z_TYPE_P(zaV))))
	{
		if (!SBGetByteArrayFromZVal(zaN, &aiN TSRMLS_CC)) RETURN_FALSE;
		if (!SBGetByteArrayFromZVal(zaG, &aiG TSRMLS_CC)) RETURN_FALSE;
		if (!SBGetByteArrayFromZVal(zaSalt, &aiSalt TSRMLS_CC)) RETURN_FALSE;
		if (!SBGetByteArrayFromZVal(zaV, &aiV TSRMLS_CC)) RETURN_FALSE;
		_err = SBSRP_SrpGetNewServerData(sUserName, (int32_t)sUserName_len, sUserPassword, (int32_t)sUserPassword_len, (TSSRPPrimeLenRaw)fPrimeLen, aiN.data, &aiN.len, aiG.data, &aiG.len, aiSalt.data, &aiSalt.len, aiV.data, &aiV.len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			SBDetachDataFromZValAndResize(zaN, &aiN TSRMLS_CC);
			SBCheckError(SBGetLastReturnBuffer(1995399686, 3, aiN.data, &aiN.len) TSRMLS_CC);
			((char *)aiN.data)[aiN.len] = 0;
			SBDetachDataFromZValAndResize(zaG, &aiG TSRMLS_CC);
			SBCheckError(SBGetLastReturnBuffer(1995399686, 4, aiG.data, &aiG.len) TSRMLS_CC);
			((char *)aiG.data)[aiG.len] = 0;
			SBDetachDataFromZValAndResize(zaSalt, &aiSalt TSRMLS_CC);
			SBCheckError(SBGetLastReturnBuffer(1995399686, 5, aiSalt.data, &aiSalt.len) TSRMLS_CC);
			((char *)aiSalt.data)[aiSalt.len] = 0;
			SBDetachDataFromZValAndResize(zaV, &aiV TSRMLS_CC);
			SBCheckError(SBGetLastReturnBuffer(1995399686, 6, aiV.data, &aiV.len) TSRMLS_CC);
			((char *)aiV.data)[aiV.len] = 0;
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBFreeArrayZValInfo(&aiN);
			SBFreeArrayZValInfo(&aiG);
			SBFreeArrayZValInfo(&aiSalt);
			SBFreeArrayZValInfo(&aiV);
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		SBSetByteArrayToZVal(&aiN, zaN);
		SBSetByteArrayToZVal(&aiG, zaG);
		SBSetByteArrayToZVal(&aiSalt, zaSalt);
		SBSetByteArrayToZVal(&aiV, zaV);
	}
	else
	{
		SBErrorExpectsArguments("(string, string, integer, &array of byte|string, &array of byte|string, &array of byte|string, &array of byte|string)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpLIntToBytesImplicit)
{
	uint32_t _err;
	zval *oX;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!", &oX, TLInt_ce_ptr) == SUCCESS)
	{
		uint8_t *u8OutResult = NULL;
		int32_t u8OutResult_len = 0;
		_err = SBSRP_SrpLIntToBytesImplicit((TLInt *)SBGetStructPointer(oX TSRMLS_CC), u8OutResult, &u8OutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			u8OutResult = emalloc(u8OutResult_len + 1);
			SBCheckError(SBGetLastReturnBuffer(-635829881, 1, u8OutResult, &u8OutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		if (NULL == u8OutResult) u8OutResult = emalloc(1);
		u8OutResult[u8OutResult_len] = 0;
		RETURN_STRINGL(u8OutResult, u8OutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("(TLInt)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpLIntToBytesPAD)
{
	long l4Len;
	uint32_t _err;
	zval *oX;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!l", &oX, TLInt_ce_ptr, &l4Len) == SUCCESS)
	{
		uint8_t *u8OutResult = NULL;
		int32_t u8OutResult_len = 0;
		_err = SBSRP_SrpLIntToBytesPAD((TLInt *)SBGetStructPointer(oX TSRMLS_CC), (int32_t)l4Len, u8OutResult, &u8OutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			u8OutResult = emalloc(u8OutResult_len + 1);
			SBCheckError(SBGetLastReturnBuffer(116353562, 2, u8OutResult, &u8OutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		if (NULL == u8OutResult) u8OutResult = emalloc(1);
		u8OutResult[u8OutResult_len] = 0;
		RETURN_STRINGL(u8OutResult, u8OutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("(TLInt, integer)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, LIntToBytes)
{
	uint32_t _err;
	zval *oI;
	if (zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!", &oI, TLInt_ce_ptr) == SUCCESS)
	{
		uint8_t *u8OutResult = NULL;
		int32_t u8OutResult_len = 0;
		_err = SBSRP_LIntToBytes((TLInt *)SBGetStructPointer(oI TSRMLS_CC), u8OutResult, &u8OutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			u8OutResult = emalloc(u8OutResult_len + 1);
			SBCheckError(SBGetLastReturnBuffer(-1101215986, 1, u8OutResult, &u8OutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		if (NULL == u8OutResult) u8OutResult = emalloc(1);
		u8OutResult[u8OutResult_len] = 0;
		RETURN_STRINGL(u8OutResult, u8OutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("(TLInt)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, LInitBytes)
{
	SBArrayZValInfo aiBA;
	zval *oI;
	zval *zaBA;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "O!z", &oI, TLInt_ce_ptr, &zaBA) == SUCCESS) && ((IS_STRING == Z_TYPE_P(zaBA)) || (IS_ARRAY == Z_TYPE_P(zaBA)) || (IS_NULL == Z_TYPE_P(zaBA))))
	{
		if (!SBGetByteArrayFromZVal(zaBA, &aiBA TSRMLS_CC)) RETURN_FALSE;
		SBCheckError(SBSRP_LInitBytes((TLInt *)SBGetStructPointer(oI TSRMLS_CC), aiBA.data, aiBA.len) TSRMLS_CC);
		SBFreeArrayZValInfo(&aiBA);
	}
	else
	{
		SBErrorExpectsArguments("(TLInt, array of byte|string|NULL)" TSRMLS_CC);
	}
}

SB_PHP_FUNCTION(SBSRP, SrpGenerateVerifier)
{
	char *sPassword;
	char *sUsername;
	int sPassword_len;
	int sUsername_len;
	SBArrayZValInfo aiGenerator;
	SBArrayZValInfo aiPrime;
	SBArrayZValInfo aiSalt;
	uint32_t _err;
	zval *zaGenerator;
	zval *zaPrime;
	zval *zaSalt;
	if ((zend_parse_parameters_ex(ZEND_PARSE_PARAMS_QUIET, ZEND_NUM_ARGS() TSRMLS_CC, "sszzz", &sUsername, &sUsername_len, &sPassword, &sPassword_len, &zaPrime, &zaGenerator, &zaSalt) == SUCCESS) && ((IS_STRING == Z_TYPE_P(zaPrime)) || (IS_ARRAY == Z_TYPE_P(zaPrime)) || (IS_NULL == Z_TYPE_P(zaPrime))) && ((IS_STRING == Z_TYPE_P(zaGenerator)) || (IS_ARRAY == Z_TYPE_P(zaGenerator)) || (IS_NULL == Z_TYPE_P(zaGenerator))) && ((IS_STRING == Z_TYPE_P(zaSalt)) || (IS_ARRAY == Z_TYPE_P(zaSalt)) || (IS_NULL == Z_TYPE_P(zaSalt))))
	{
		uint8_t *u8OutResult = NULL;
		int32_t u8OutResult_len = 0;
		if (!SBGetByteArrayFromZVal(zaPrime, &aiPrime TSRMLS_CC)) RETURN_FALSE;
		if (!SBGetByteArrayFromZVal(zaGenerator, &aiGenerator TSRMLS_CC)) RETURN_FALSE;
		if (!SBGetByteArrayFromZVal(zaSalt, &aiSalt TSRMLS_CC)) RETURN_FALSE;
		_err = SBSRP_SrpGenerateVerifier(sUsername, (int32_t)sUsername_len, sPassword, (int32_t)sPassword_len, aiPrime.data, aiPrime.len, aiGenerator.data, aiGenerator.len, aiSalt.data, aiSalt.len, u8OutResult, &u8OutResult_len);
		if (SB_ERROR_INSUFFICIENT_BUFFER_SIZE == _err)
		{
			u8OutResult = emalloc(u8OutResult_len + 1);
			SBCheckError(SBGetLastReturnBuffer(-503293982, 5, u8OutResult, &u8OutResult_len) TSRMLS_CC);
		}
		else
		if (_err != SB_ERROR_OK)
		{
			SBThrowException(_err TSRMLS_CC);
			RETURN_FALSE;
		}
	
		SBFreeArrayZValInfo(&aiPrime);
		SBFreeArrayZValInfo(&aiGenerator);
		SBFreeArrayZValInfo(&aiSalt);
		if (NULL == u8OutResult) u8OutResult = emalloc(1);
		u8OutResult[u8OutResult_len] = 0;
		RETURN_STRINGL(u8OutResult, u8OutResult_len, 0);
	}
	else
	{
		SBErrorExpectsArguments("(string, string, array of byte|string|NULL, array of byte|string|NULL, array of byte|string|NULL)" TSRMLS_CC);
	}
}

void Register_SBSRP_Constants(int module_number TSRMLS_DC)
{
	SB_REGISTER_LONG_CONSTANT(SBSRP, TSRPPrimesGen_1536, SB_TSRPPrimesGen_1536, SB_TSRPPrimesGen_1536);
	SB_REGISTER_LONG_CONSTANT(SBSRP, TSRPPrimesGen_2048, SB_TSRPPrimesGen_2048, SB_TSRPPrimesGen_2048);
	SB_REGISTER_LONG_CONSTANT(SBSRP, TSRPPrimesGen_3072, SB_TSRPPrimesGen_3072, SB_TSRPPrimesGen_3072);
	SB_REGISTER_LONG_CONSTANT(SBSRP, TSRPPrimesGen_4096, SB_TSRPPrimesGen_4096, SB_TSRPPrimesGen_4096);
	SB_REGISTER_LONG_CONSTANT(SBSRP, TSRPPrimesGen_6144, SB_TSRPPrimesGen_6144, SB_TSRPPrimesGen_6144);
	SB_REGISTER_LONG_CONSTANT(SBSRP, TSRPPrimesGen_8192, SB_TSRPPrimesGen_8192, SB_TSRPPrimesGen_8192);
}

void Register_SBSRP_Enum_Flags(TSRMLS_D)
{
	zend_class_entry ce;
	INIT_CLASS_ENTRY(ce, "TSSRPPrimeLen", NULL);
	TSSRPPrimeLen_ce_ptr = zend_register_internal_class_ex(&ce, TSBBaseEnum_ce_ptr, NULL TSRMLS_CC);
	TSSRPPrimeLen_ce_ptr->ce_flags |= ZEND_ACC_FINAL_CLASS;
	SB_DECLARE_CLASS_LONG_CONST(TSSRPPrimeLen_ce_ptr, "sr1024", 0)
	SB_DECLARE_CLASS_LONG_CONST(TSSRPPrimeLen_ce_ptr, "sr1536", 1)
	SB_DECLARE_CLASS_LONG_CONST(TSSRPPrimeLen_ce_ptr, "sr2048", 2)
	SB_DECLARE_CLASS_LONG_CONST(TSSRPPrimeLen_ce_ptr, "sr3072", 3)
	SB_DECLARE_CLASS_LONG_CONST(TSSRPPrimeLen_ce_ptr, "sr4096", 4)
	SB_DECLARE_CLASS_LONG_CONST(TSSRPPrimeLen_ce_ptr, "sr6144", 5)
	SB_DECLARE_CLASS_LONG_CONST(TSSRPPrimeLen_ce_ptr, "sr8192", 6)
	
	INIT_CLASS_ENTRY(ce, "TSBSRPCredentialPrimeLen", NULL);
	TSBSRPCredentialPrimeLen_ce_ptr = zend_register_internal_class_ex(&ce, TSBBaseEnum_ce_ptr, NULL TSRMLS_CC);
	TSBSRPCredentialPrimeLen_ce_ptr->ce_flags |= ZEND_ACC_FINAL_CLASS;
	SB_DECLARE_CLASS_LONG_CONST(TSBSRPCredentialPrimeLen_ce_ptr, "srppl1024", 0)
	SB_DECLARE_CLASS_LONG_CONST(TSBSRPCredentialPrimeLen_ce_ptr, "srpp1536", 1)
	SB_DECLARE_CLASS_LONG_CONST(TSBSRPCredentialPrimeLen_ce_ptr, "srpp2048", 2)
	SB_DECLARE_CLASS_LONG_CONST(TSBSRPCredentialPrimeLen_ce_ptr, "srpp3072", 3)
	SB_DECLARE_CLASS_LONG_CONST(TSBSRPCredentialPrimeLen_ce_ptr, "srpp4096", 4)
	SB_DECLARE_CLASS_LONG_CONST(TSBSRPCredentialPrimeLen_ce_ptr, "srpp6144", 5)
	SB_DECLARE_CLASS_LONG_CONST(TSBSRPCredentialPrimeLen_ce_ptr, "srpp8192", 6)
}

