#ifndef __INC_SBASN1TREE
#define __INC_SBASN1TREE

#if _MSC_VER > 1000
#  pragma once
#endif // _MSC_VER > 1000

#include "sbdefs.h"
#include "sbphpcore.h"
#include "sbsystem.h"
#include "sbstreams.h"
#include "sbtypes.h"
#include "sbutils.h"
#include "sbstrutils.h"
#include "sbconstants.h"
#include "sbasn1.h"

#pragma pack(push, 1)

#ifdef __cplusplus
namespace SecureBlackbox {
#endif

#ifdef __cplusplus
extern "C" {
#endif
#define SB_ASN1_BOOLEAN 	1
#define SB_ASN1_INTEGER 	2
#define SB_ASN1_BITSTRING 	3
#define SB_ASN1_OCTETSTRING 	4
#define SB_ASN1_NULL 	5
#define SB_ASN1_OBJECT 	6
#define SB_ASN1_REAL 	9
#define SB_ASN1_ENUMERATED 	10
#define SB_ASN1_UTF8STRING 	12
#define SB_ASN1_NUMERICSTR 	18
#define SB_ASN1_PRINTABLESTRING 	19
#define SB_ASN1_T61STRING 	20
#define SB_ASN1_TELETEXSTRING 	20
#define SB_ASN1_VIDEOTEXSTRING 	21
#define SB_ASN1_IA5STRING 	22
#define SB_ASN1_UTCTIME 	23
#define SB_ASN1_GENERALIZEDTIME 	24
#define SB_ASN1_GRAPHICSTRING 	25
#define SB_ASN1_VISIBLESTRING 	26
#define SB_ASN1_GENERALSTRING 	27
#define SB_ASN1_UNIVERSALSTRING 	28
#define SB_ASN1_BMPSTRING 	30
#define SB_ASN1_SEQUENCE 	48
#define SB_ASN1_SET 	49
#define SB_ASN1_A0_PRIMITIVE 	128
#define SB_ASN1_A0 	160
#define SB_ASN1_A1_PRIMITIVE 	129
#define SB_ASN1_A1 	161
#define SB_ASN1_A2_PRIMITIVE 	130
#define SB_ASN1_A2 	162
#define SB_ASN1_A3_PRIMITIVE 	131
#define SB_ASN1_A3 	163
#define SB_ASN1_A4_PRIMITIVE 	132
#define SB_ASN1_A4 	164
#define SB_ASN1_A5_PRIMITIVE 	133
#define SB_ASN1_A5 	165
#define SB_ASN1_A6_PRIMITIVE 	134
#define SB_ASN1_A6 	166
#define SB_ASN1_A7_PRIMITIVE 	135
#define SB_ASN1_A7 	167
#define SB_ASN1_A8_PRIMITIVE 	136
#define SB_ASN1_A8 	168
#define SB_ASN1_A9_PRIMITIVE 	137
#define SB_ASN1_A9 	169
#define SB_ASN1_CONSTRAINED_FLAG 	32

typedef TElClassHandle TElASN1CustomTagHandle;

typedef TElClassHandle TElASN1DataSourceHandle;

typedef TElClassHandle TElASN1SimpleTagHandle;

typedef TElClassHandle TElASN1ConstrainedTagHandle;

typedef uint8_t TSBASN1DataSourceTypeRaw;

typedef enum
{
	dstBuffer = 0,
	dstStream = 1,
	dstVirtual = 2
} TSBASN1DataSourceType;

typedef void (SB_CALLBACK *TSBASN1VirtualDataNeededEvent)(void * _ObjectData, TObjectHandle Sender, int64_t StartIndex, void * Buffer, int32_t MaxSize, int32_t * Read);

typedef uint8_t TSBASN1StreamAccessRaw;

typedef enum
{
	saStoreStream = 0
} TSBASN1StreamAccess;

#ifdef SB_USE_CLASS_TELASN1CUSTOMTAG
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_LoadFromBuffer(TElASN1CustomTagHandle _Handle, void * Buffer, int32_t Size, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_SaveToBuffer(TElASN1CustomTagHandle _Handle, void * Buffer, int32_t * Size, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_CheckType(TElASN1CustomTagHandle _Handle, uint8_t TagId, int8_t Constrained, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_LoadFromStream(TElASN1CustomTagHandle _Handle, TStreamHandle Stream, int64_t Count, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_SaveToStream(TElASN1CustomTagHandle _Handle, TStreamHandle Stream);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_TagId(TElASN1CustomTagHandle _Handle, uint8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_set_TagId(TElASN1CustomTagHandle _Handle, uint8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_UndefSize(TElASN1CustomTagHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_set_UndefSize(TElASN1CustomTagHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_WriteHeader(TElASN1CustomTagHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_set_WriteHeader(TElASN1CustomTagHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_IsConstrained(TElASN1CustomTagHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_TagNum(TElASN1CustomTagHandle _Handle, uint8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_TagOffset(TElASN1CustomTagHandle _Handle, int64_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_TagSize(TElASN1CustomTagHandle _Handle, int64_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_TagHeaderSize(TElASN1CustomTagHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_TagContentSize(TElASN1CustomTagHandle _Handle, int64_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_get_Depth(TElASN1CustomTagHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1CustomTag_Create(TElASN1CustomTagHandle * OutResult);
#endif /* SB_USE_CLASS_TELASN1CUSTOMTAG */

#ifdef SB_USE_CLASS_TELASN1DATASOURCE
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_Init(TElASN1DataSourceHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_Init_1(TElASN1DataSourceHandle _Handle, TStreamHandle Stream, int64_t Offset, int64_t Size);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_Init_2(TElASN1DataSourceHandle _Handle, TStreamHandle Stream, int8_t UnknownSize);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_Init_3(TElASN1DataSourceHandle _Handle, void * Buffer, int32_t Size);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_InitVirtual(TElASN1DataSourceHandle _Handle, int64_t Size);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_Read(TElASN1DataSourceHandle _Handle, void * Buffer, int32_t Size, int64_t Offset, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_Clone(TElASN1DataSourceHandle _Handle, TElASN1DataSourceHandle Dest);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_CloneVirtual(TElASN1DataSourceHandle _Handle, TElASN1DataSourceHandle Dest);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_ToBuffer(TElASN1DataSourceHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_get_Size(TElASN1DataSourceHandle _Handle, int64_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_get_UnknownSize(TElASN1DataSourceHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_get_SkipVirtualData(TElASN1DataSourceHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_set_SkipVirtualData(TElASN1DataSourceHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_get_SourceType(TElASN1DataSourceHandle _Handle, TSBASN1DataSourceTypeRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_get_OnVirtualDataNeeded(TElASN1DataSourceHandle _Handle, TSBASN1VirtualDataNeededEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_set_OnVirtualDataNeeded(TElASN1DataSourceHandle _Handle, TSBASN1VirtualDataNeededEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DataSource_Create(TObjectHandle * OutResult);
#endif /* SB_USE_CLASS_TELASN1DATASOURCE */

#ifdef SB_USE_CLASS_TELASN1SIMPLETAG
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_CreateInstance(TElASN1SimpleTagHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_CreateInstance_1(TElASN1SimpleTagHandle _Handle, TElASN1SimpleTagHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_LoadFromBuffer(TElASN1SimpleTagHandle _Handle, void * Buffer, int32_t Size, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_SaveToBuffer(TElASN1SimpleTagHandle _Handle, void * Buffer, int32_t * Size, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_LoadFromStream(TElASN1SimpleTagHandle _Handle, TStreamHandle Stream, int64_t Count, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_SaveToStream(TElASN1SimpleTagHandle _Handle, TStreamHandle Stream);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_get_Content(TElASN1SimpleTagHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_set_Content(TElASN1SimpleTagHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_get_DataSource(TElASN1SimpleTagHandle _Handle, TElASN1DataSourceHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_get_FragmentSize(TElASN1SimpleTagHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_set_FragmentSize(TElASN1SimpleTagHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_get_OnContentWriteBegin(TElASN1SimpleTagHandle _Handle, TNotifyEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_set_OnContentWriteBegin(TElASN1SimpleTagHandle _Handle, TNotifyEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_get_OnContentWriteEnd(TElASN1SimpleTagHandle _Handle, TNotifyEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_set_OnContentWriteEnd(TElASN1SimpleTagHandle _Handle, TNotifyEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElASN1SimpleTag_Create(TElASN1SimpleTagHandle * OutResult);
#endif /* SB_USE_CLASS_TELASN1SIMPLETAG */

#ifdef SB_USE_CLASS_TELASN1CONSTRAINEDTAG
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_CreateInstance(TElASN1ConstrainedTagHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_CreateInstance_1(TElASN1ConstrainedTagHandle _Handle, TElASN1ConstrainedTagHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_LoadFromBuffer(TElASN1ConstrainedTagHandle _Handle, void * Buffer, int32_t Size, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_LoadFromBufferSingle(TElASN1ConstrainedTagHandle _Handle, void * Buffer, int32_t Size, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_SaveToBuffer(TElASN1ConstrainedTagHandle _Handle, void * Buffer, int32_t * Size, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_SaveContentToBuffer(TElASN1ConstrainedTagHandle _Handle, void * Buffer, int32_t * Size, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_LoadFromStream(TElASN1ConstrainedTagHandle _Handle, TStreamHandle Stream, int64_t Count, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_LoadFromStreamSingle(TElASN1ConstrainedTagHandle _Handle, TStreamHandle Stream, int64_t Count, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_SaveToStream(TElASN1ConstrainedTagHandle _Handle, TStreamHandle Stream);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_AddField(TElASN1ConstrainedTagHandle _Handle, int8_t Constrained, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_RemoveField(TElASN1ConstrainedTagHandle _Handle, int32_t Index, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_GetField(TElASN1ConstrainedTagHandle _Handle, int32_t Index, TElASN1CustomTagHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_Clear(TElASN1ConstrainedTagHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_get_Count(TElASN1ConstrainedTagHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_get_MaxSimpleTagLength(TElASN1ConstrainedTagHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_set_MaxSimpleTagLength(TElASN1ConstrainedTagHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_get_StreamAccess(TElASN1ConstrainedTagHandle _Handle, TSBASN1StreamAccessRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_set_StreamAccess(TElASN1ConstrainedTagHandle _Handle, TSBASN1StreamAccessRaw Value);
SB_IMPORT uint32_t SB_APIENTRY TElASN1ConstrainedTag_Create(TElASN1ConstrainedTagHandle * OutResult);
#endif /* SB_USE_CLASS_TELASN1CONSTRAINEDTAG */

#ifdef __cplusplus
};	/* extern "C" */
#endif

extern zend_class_entry *TElASN1CustomTag_ce_ptr;
extern zend_class_entry *TElASN1DataSource_ce_ptr;
extern zend_class_entry *TElASN1SimpleTag_ce_ptr;
extern zend_class_entry *TElASN1ConstrainedTag_ce_ptr;

void SB_CALLBACK TSBASN1VirtualDataNeededEventRaw(void * _ObjectData, TObjectHandle Sender, int64_t StartIndex, void * Buffer, int32_t MaxSize, int32_t * Read);
void Register_TElASN1CustomTag(TSRMLS_D);
void Register_TElASN1DataSource(TSRMLS_D);
void Register_TElASN1SimpleTag(TSRMLS_D);
void Register_TElASN1ConstrainedTag(TSRMLS_D);
SB_PHP_FUNCTION(SBASN1Tree, asymWriteInteger);
SB_PHP_FUNCTION(SBASN1Tree, ASN1ReadInteger);
SB_PHP_FUNCTION(SBASN1Tree, ASN1ReadInteger64);
SB_PHP_FUNCTION(SBASN1Tree, ASN1WriteInteger);
SB_PHP_FUNCTION(SBASN1Tree, ASN1WriteInteger64);
SB_PHP_FUNCTION(SBASN1Tree, ASN1ReadSimpleValue);
SB_PHP_FUNCTION(SBASN1Tree, ASN1WriteTagAndLength);
SB_PHP_FUNCTION(SBASN1Tree, ASN1ReadBoolean);
SB_PHP_FUNCTION(SBASN1Tree, ASN1WriteBoolean);
SB_PHP_FUNCTION(SBASN1Tree, ASN1ReadString);
SB_PHP_FUNCTION(SBASN1Tree, ASN1ReadStringAnsi);
SB_PHP_FUNCTION(SBASN1Tree, FormatAttributeValue);
SB_PHP_FUNCTION(SBASN1Tree, UnformatAttributeValue);
SB_PHP_FUNCTION(SBASN1Tree, GetMaxASN1TreeDepth);
SB_PHP_FUNCTION(SBASN1Tree, SetMaxASN1TreeDepth);
SB_PHP_FUNCTION(SBASN1Tree, GetMaxASN1BufferLength);
SB_PHP_FUNCTION(SBASN1Tree, SetMaxASN1BufferLength);
void Register_SBASN1Tree_Constants(int module_number TSRMLS_DC);
void Register_SBASN1Tree_Enum_Flags(TSRMLS_D);

#ifdef __cplusplus
extern "C" {
#endif

#ifdef SB_USE_GLOBAL_PROCS_ASN1TREE
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_asymWriteInteger(TElASN1SimpleTagHandle Tag, void * Buffer, int32_t Size);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1ReadInteger(TElASN1SimpleTagHandle Tag, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1ReadInteger64(TElASN1SimpleTagHandle Tag, int64_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1WriteInteger(TElASN1SimpleTagHandle Tag, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1WriteInteger64(TElASN1SimpleTagHandle Tag, int64_t Value);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1ReadSimpleValue(const uint8_t pData[], int32_t szData, int32_t * TagID, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1WriteTagAndLength(int32_t Tag, int64_t Len, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1ReadBoolean(TElASN1SimpleTagHandle Tag, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1WriteBoolean(TElASN1SimpleTagHandle Tag, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1ReadString(const uint8_t pData[], int32_t szData, int32_t TagId, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_ASN1ReadStringAnsi(const uint8_t pData[], int32_t szData, int32_t TagId, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_FormatAttributeValue(int32_t TagID, const uint8_t pValue[], int32_t szValue, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_UnformatAttributeValue(const uint8_t pValue[], int32_t szValue, int32_t * TagID, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_GetMaxASN1TreeDepth(int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_SetMaxASN1TreeDepth(int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_GetMaxASN1BufferLength(int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY SBASN1Tree_SetMaxASN1BufferLength(int32_t Value);
#endif /* SB_USE_GLOBAL_PROCS_ASN1TREE */

#ifdef __cplusplus
};	/* extern "C" */
#endif

#ifdef __cplusplus
};	/* namespace SecureBlackbox */
#endif

#pragma pack(pop)

#endif  /* __INC_SBASN1TREE */

