#ifndef __INC_SBCMS
#define __INC_SBCMS

#if _MSC_VER > 1000
#  pragma once
#endif // _MSC_VER > 1000

#include "sbdefs.h"
#include "sbphpcore.h"
#include "sbsystem.h"
#include "sbstreams.h"
#include "sbconstants.h"
#include "sbtypes.h"
#include "sbutils.h"
#include "sbstrutils.h"
#include "sbrandom.h"
#include "sbcryptoprov.h"
#include "sbcustomcertstorage.h"
#include "sbtspclient.h"
#include "sbhashfunction.h"
#include "sbcustomcrypto.h"
#include "sbsymmetriccrypto.h"
#include "sbpublickeycrypto.h"
#include "sbcrl.h"
#include "sbcrlstorage.h"
#include "sbx509.h"
#include "sbx509ext.h"
#include "sbpkcs7utils.h"
#include "sbpkcs7.h"
#include "sbasn1.h"
#include "sbasn1tree.h"
#include "sbpkicommon.h"
#include "sbocspcommon.h"
#include "sbocspclient.h"
#include "sbocspstorage.h"
#include "sbcmsutils.h"
#include "sbalgorithmidentifier.h"
#include "sbcertvalidator.h"
#include "sbdc.h"
#include "sbdcdef.h"
#include "sbdcpkiconstants.h"
#include "sbrdn.h"

#pragma pack(push, 1)

#ifdef __cplusplus
namespace SecureBlackbox {
#endif

#ifdef __cplusplus
extern "C" {
#endif
#define SB_ERROR_FACILITY_CMS 	9216
#define SB_CMS_ERROR_INVALID_CERT_REFS 	9217
#define SB_CMS_ERROR_INVALID_REVOC_REFS 	9218
#define SB_CMS_ERROR_INVALID_REVOC_VALS 	9219
#define SB_CMS_ERROR_INVALID_CRL_ID 	9220
#define SB_CMS_ERROR_INVALID_OCSP_ID 	9221
#define SB_CMS_ERROR_INVALID_SIGNER_ID 	9222
#define SB_CMS_ERROR_INVALID_ALG_ID 	9223
#define SB_CMS_ERROR_INVALID_PARAMETERS 	9224
#define SB_CMS_ERROR_INVALID_HASH 	9225
#define SB_CMS_ERROR_INVALID_SIGN_STRUCT 	9226
#define SB_CMS_ERROR_INVALID_COUNTERSIGN 	9227
#define SB_CMS_ERROR_INVALID_TIMESTAMP_TYPE 	9228
#define SB_CMS_ERROR_INVALID_TIMESTAMP 	9229
#define SB_CMS_ERROR_INVALID_TIMESTAMP_CMS 	9230
#define SB_CMS_ERROR_INVALID_TIMESTAMP_ASN1 	9231
#define SB_CMS_ERROR_INVALID_PKCS7_IN_DATA 	9232
#define SB_CMS_ERROR_INVALID_KEY_PARAMS 	9233
#define SB_CMS_ERROR_INVALID_POLICY_QUAL 	9234
#define SB_CMS_ERROR_INVALID_SIG_POLICY 	9235
#define SB_CMS_ERROR_INVALID_CONTENT_HINTS 	9236
#define SB_CMS_ERROR_INVALID_CONTENT_REFS 	9237
#define SB_CMS_ERROR_INVALID_SIGNER_LOCATION 	9238
#define SB_CMS_ERROR_INVALID_SIGNING_CERT 	9239
#define SB_CMS_ERROR_INVALID_SIGNER_ATTRS 	9240
#define SB_CMS_ERROR_INVALID_ATS_HASH_INDEX 	9241
#define SB_CMS_ERROR_UNSUPPORTED_ALGORITHM 	9248
#define SB_CMS_ERROR_NO_PRIVATE_KEY 	9249
#define SB_CMS_ERROR_NO_DECRYPTION_KEY 	9250
#define SB_CMS_ERROR_NO_SIGNED_DATA 	9251
#define SB_CMS_ERROR_NO_SIGNING_CERT 	9252
#define SB_CMS_ERROR_NO_RECIPIENT_CERT 	9253
#define SB_CMS_ERROR_WRONG_CONTENT_TYPE 	9254
#define SB_CMS_ERROR_TIMESTAMPING_FAILED 	9265
#define SB_CMS_ERROR_REMOTE_SIGN_FAILED 	9266
#define SB_CMS_ERROR_DECRYPTION_FAILED 	9267
#define SB_CMS_ERROR_UNEXPECTED_END_OF_DATA 	9281
#define SB_CMS_ERROR_UNEXPECTED_ELEMENT_IN_CHAIN 	9282
#define SB_CMS_ERROR_DUPLICATE_ELEMENT_IN_CHAIN 	9283
#define SB_CMS_ERROR_CHAIN_INCOMPLETE 	9284
#define SB_CMS_ERROR_INCONSISTENT_TIMESTAMP_STATE 	9285
#define SB_CMS_ERROR_ASN1_SERIALIZATION_FAILED 	9286
#define SB_CMS_ERROR_CANT_SAVE_TO_SOURCE 	9287
#define SB_CMS_ERROR_BUFFER_TOO_SMALL 	9288
#define SB_CMS_ERROR_BAD_TEMPLATE 	9289
#define SB_CMS_ERROR_INVALID_SIGNATURE_POLICY_STORE 	9290
#define SB_CMS_ERROR_UNSUPPORTED_SIGNING_MODE 	9291

typedef TElClassHandle TElASN1DirectoryStringHandle;

typedef TElClassHandle TElCMSPropertyHandle;

typedef TElClassHandle TElCMSHashHandle;

typedef TElClassHandle TElCMSSignerIdentifierHandle;

typedef TElClassHandle TElCMSCertificateRefsHandle;

typedef TElClassHandle TElCMSCRLIdentifierHandle;

typedef TElClassHandle TElCMSCRLValidatedIDHandle;

typedef TElClassHandle TElCMSOCSPIdentifierHandle;

typedef TElClassHandle TElCMSOCSPResponsesIDHandle;

typedef TElClassHandle TElCMSRevocationRefHandle;

typedef TElClassHandle TElCMSRevocationRefsHandle;

typedef TElClassHandle TElCMSRevocationValuesHandle;

typedef TElClassHandle TElCMSContentHandle;

typedef TElClassHandle TElCMSSigPolicyQualifierHandle;

typedef TElClassHandle TElCMSSignaturePolicyHandle;

typedef TElClassHandle TElCMSContentHintsHandle;

typedef TElClassHandle TElCMSContentReferenceHandle;

typedef TElClassHandle TElCMSCommitmentTypeIndicationHandle;

typedef TElClassHandle TElCMSSignerLocationHandle;

typedef TElClassHandle TElCMSSigningCertificateHandle;

typedef TElClassHandle TElCMSSignerAttributesHandle;

typedef TElClassHandle TElCMSSignaturePolicyStoreHandle;

typedef TElClassHandle TElCMSSignatureHandle;

typedef TElClassHandle TElCMSTimestampHandle;

typedef TElClassHandle TElATSHashIndexProcessorHandle;

typedef TElClassHandle TElSignedCMSMessageHandle;

typedef TElClassHandle TElCMSContentHashHandle;

typedef TElClassHandle TElCMSMessageHandle;

typedef TElClassHandle TElSignedAndEnvelopedCMSMessageHandle;

typedef uint8_t TSBASN1DirectoryStringEncodingRaw;

typedef enum
{
	dseTeletexString = 0,
	dsePrintableString = 1,
	dseUniversalString = 2,
	dseUTF8String = 3,
	dseBMPString = 4
} TSBASN1DirectoryStringEncoding;

typedef uint8_t TSBCMSRevocationRefsWriteModeRaw;

typedef enum
{
	rrwmWriteAll = 0,
	rrwmWriteCRLIfPresent = 1,
	rrwmWriteOCSPIfPresent = 2
} TSBCMSRevocationRefsWriteMode;

typedef uint8_t TSBCMSSignerIdentifierTypeRaw;

typedef enum
{
	sitESSCertID = 0,
	sitOtherCertID = 1,
	sitESSCertIDV2 = 2
} TSBCMSSignerIdentifierType;

typedef uint8_t TSBSigPolicyQualifierTypeRaw;

typedef enum
{
	spqtURI = 0,
	spqtUserNotice = 1,
	spqtCustom = 2
} TSBSigPolicyQualifierType;

typedef uint8_t TSBCMSSigningCertificateTypeRaw;

typedef enum
{
	sctESSSigningCertificate = 0,
	sctOtherSigningCertificate = 1,
	sctESSSigningCertificateV2 = 2
} TSBCMSSigningCertificateType;

typedef uint8_t TSBCMSSignerAttributesVersionRaw;

typedef enum
{
	saVersion1 = 0,
	saVersion2 = 1
} TSBCMSSignerAttributesVersion;

typedef uint8_t TSBCMSSignaturePolicyStoreTypeRaw;

typedef enum
{
	pstDocument = 0,
	pstURI = 1
} TSBCMSSignaturePolicyStoreType;

typedef uint8_t TSBCMSTimestampTypeRaw;

typedef enum
{
	cvtGeneric = 0,
	cvtESC = 1,
	cvtCertsAndCRLs = 2,
	cvtArchive = 3,
	cvtArchive2 = 4,
	cvtArchive3 = 5,
	cvtContent = 6
} TSBCMSTimestampType;

typedef uint8_t TSBCMSItemTypeRaw;

typedef enum
{
	citCertificate = 0,
	citCRL = 1,
	citOCSPResponse = 2,
	citTimestamp = 3
} TSBCMSItemType;

typedef void (SB_CALLBACK *TSBCMSBeforeSignEvent)(void * _ObjectData, TObjectHandle Sender, TElPublicKeyCryptoHandle Crypto);

typedef void (SB_CALLBACK *TSBCMSBeforeVerifyEvent)(void * _ObjectData, TObjectHandle Sender, TElPublicKeyCryptoHandle Crypto);

typedef void (SB_CALLBACK *TSBCMSCertificateTrustedEvent)(void * _ObjectData, TObjectHandle Sender, TElX509CertificateHandle Cert, int8_t * Trusted);

typedef void (SB_CALLBACK *TSBCMSItemValidatedEvent)(void * _ObjectData, TObjectHandle Sender, TSBCMSItemTypeRaw ItemType, TObjectHandle Item, int8_t * Valid);

typedef void (SB_CALLBACK *TSBCMSCRLNeededEvent)(void * _ObjectData, TObjectHandle Sender, TElCMSCRLValidatedIDHandle ID, TElX509CertificateHandle Cert, TElCertificateRevocationListHandle * Crl);

typedef void (SB_CALLBACK *TSBCMSOCSPResponseNeededEvent)(void * _ObjectData, TObjectHandle Sender, TElCMSOCSPResponsesIDHandle ID, TElX509CertificateHandle Cert, TElOCSPResponseHandle * Ocsp);

typedef uint8_t TSBCMSSigningOptionRaw;

typedef enum
{
	csoInsertMessageDigests = 0,
	csoInsertSigningTime = 1,
	csoInsertContentType = 2,
	csoUseGeneralizedTimeFormat = 3,
	csoIncludeCertToMessage = 4,
	csoIncludeCertToAttributes = 5,
	csoImzagerMIMCompatibility = 6,
	csoForceSigningCertificateV2Usage = 7,
	csoAnnexKArchiveTimestampV2Mode = 8,
	csoDigitPADSSCompatibility = 9,
	csoCompareRDNAsStrings = 10,
	csoIgnoreDERReqInArchiveTimestamps = 11,
	csoSkipEnvContentInfoOnSigArchival = 12,
	csoUseGenericSigAlgorithmOIDs = 13,
	csoUsePlainContentForTimestampHashes = 14,
	csoUseATSHashIndexV1 = 15,
	csoCheckATSHashIndexElements = 16
} TSBCMSSigningOption;

typedef uint32_t TSBCMSSigningOptionsRaw;

typedef enum 
{
	f_csoInsertMessageDigests = 1,
	f_csoInsertSigningTime = 2,
	f_csoInsertContentType = 4,
	f_csoUseGeneralizedTimeFormat = 8,
	f_csoIncludeCertToMessage = 16,
	f_csoIncludeCertToAttributes = 32,
	f_csoImzagerMIMCompatibility = 64,
	f_csoForceSigningCertificateV2Usage = 128,
	f_csoAnnexKArchiveTimestampV2Mode = 256,
	f_csoDigitPADSSCompatibility = 512,
	f_csoCompareRDNAsStrings = 1024,
	f_csoIgnoreDERReqInArchiveTimestamps = 2048,
	f_csoSkipEnvContentInfoOnSigArchival = 4096,
	f_csoUseGenericSigAlgorithmOIDs = 8192,
	f_csoUsePlainContentForTimestampHashes = 16384,
	f_csoUseATSHashIndexV1 = 32768,
	f_csoCheckATSHashIndexElements = 65536
} TSBCMSSigningOptions;

typedef void (SB_CALLBACK *TSBCMSRemoteSignCallback)(void * _ObjectData, TObjectHandle Sender, TObjectHandle Param, const uint8_t pHash[], int32_t szHash, uint8_t pSignedHash[], int32_t * szSignedHash, int8_t * OutResult);

typedef uint8_t TSBCMSSignatureStateRaw;

typedef enum
{
	cssSigned = 0,
	cssUnsigned = 1
} TSBCMSSignatureState;

#ifdef SB_USE_CLASS_TELASN1DIRECTORYSTRING
SB_IMPORT uint32_t SB_APIENTRY TElASN1DirectoryString_get_Value(TElASN1DirectoryStringHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DirectoryString_set_Value(TElASN1DirectoryStringHandle _Handle, const char * pcValue, int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DirectoryString_get_Encoding(TElASN1DirectoryStringHandle _Handle, TSBASN1DirectoryStringEncodingRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DirectoryString_set_Encoding(TElASN1DirectoryStringHandle _Handle, TSBASN1DirectoryStringEncodingRaw Value);
SB_IMPORT uint32_t SB_APIENTRY TElASN1DirectoryString_Create(TElASN1DirectoryStringHandle * OutResult);
#endif /* SB_USE_CLASS_TELASN1DIRECTORYSTRING */

#ifdef SB_USE_CLASS_TELCMSPROPERTY
SB_IMPORT uint32_t SB_APIENTRY TElCMSProperty_get_Included(TElCMSPropertyHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSProperty_set_Included(TElCMSPropertyHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSProperty_get_Signed(TElCMSPropertyHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSProperty_set_Signed(TElCMSPropertyHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSProperty_get_RecalculateValue(TElCMSPropertyHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSProperty_set_RecalculateValue(TElCMSPropertyHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSProperty_get_RawValue(TElCMSPropertyHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSProperty_Create(TElCMSPropertyHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSPROPERTY */

#ifdef SB_USE_CLASS_TELCMSHASH
SB_IMPORT uint32_t SB_APIENTRY TElCMSHash_Clear(TElCMSHashHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSHash_get_HashAlgorithm(TElCMSHashHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSHash_set_HashAlgorithm(TElCMSHashHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSHash_get_HashAlgorithmParams(TElCMSHashHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSHash_set_HashAlgorithmParams(TElCMSHashHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSHash_get_HashValue(TElCMSHashHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSHash_set_HashValue(TElCMSHashHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSHash_Create(TElCMSHashHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSHASH */

#ifdef SB_USE_CLASS_TELCMSSIGNERIDENTIFIER
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerIdentifier_Import(TElCMSSignerIdentifierHandle _Handle, TElX509CertificateHandle Cert, int32_t HashAlgorithm);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerIdentifier_Clear(TElCMSSignerIdentifierHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerIdentifier_Corresponds(TElCMSSignerIdentifierHandle _Handle, TElX509CertificateHandle Cert, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerIdentifier_get_Hash(TElCMSSignerIdentifierHandle _Handle, TElCMSHashHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerIdentifier_get_Issuer(TElCMSSignerIdentifierHandle _Handle, TElGeneralNamesHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerIdentifier_get_SerialNumber(TElCMSSignerIdentifierHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerIdentifier_get_SignerIdentifierType(TElCMSSignerIdentifierHandle _Handle, TSBCMSSignerIdentifierTypeRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerIdentifier_set_SignerIdentifierType(TElCMSSignerIdentifierHandle _Handle, TSBCMSSignerIdentifierTypeRaw Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerIdentifier_Create(TElCMSSignerIdentifierHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSSIGNERIDENTIFIER */

#ifdef SB_USE_CLASS_TELCMSCERTIFICATEREFS
SB_IMPORT uint32_t SB_APIENTRY TElCMSCertificateRefs_Add(TElCMSCertificateRefsHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCertificateRefs_Remove(TElCMSCertificateRefsHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCertificateRefs_Clear(TElCMSCertificateRefsHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCertificateRefs_get_CertIDs(TElCMSCertificateRefsHandle _Handle, int32_t Index, TElCMSSignerIdentifierHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCertificateRefs_get_CertIDCount(TElCMSCertificateRefsHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCertificateRefs_Create(TElCMSCertificateRefsHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSCERTIFICATEREFS */

#ifdef SB_USE_CLASS_TELCMSCRLIDENTIFIER
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLIdentifier_Clear(TElCMSCRLIdentifierHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLIdentifier_get_Issuer(TElCMSCRLIdentifierHandle _Handle, TElRelativeDistinguishedNameHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLIdentifier_get_Time(TElCMSCRLIdentifierHandle _Handle, int64_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLIdentifier_set_Time(TElCMSCRLIdentifierHandle _Handle, int64_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLIdentifier_get_Number(TElCMSCRLIdentifierHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLIdentifier_set_Number(TElCMSCRLIdentifierHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLIdentifier_Create(TElCMSCRLIdentifierHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSCRLIDENTIFIER */

#ifdef SB_USE_CLASS_TELCMSCRLVALIDATEDID
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLValidatedID_Import(TElCMSCRLValidatedIDHandle _Handle, TElCertificateRevocationListHandle CRL, int32_t HashAlgorithm);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLValidatedID_CRLMatches(TElCMSCRLValidatedIDHandle _Handle, TElCertificateRevocationListHandle CRL, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLValidatedID_get_CRLHash(TElCMSCRLValidatedIDHandle _Handle, TElCMSHashHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLValidatedID_get_CRLIdentifier(TElCMSCRLValidatedIDHandle _Handle, TElCMSCRLIdentifierHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCRLValidatedID_Create(TElCMSCRLValidatedIDHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSCRLVALIDATEDID */

#ifdef SB_USE_CLASS_TELCMSOCSPIDENTIFIER
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPIdentifier_Assign(TElCMSOCSPIdentifierHandle _Handle, TElCMSOCSPIdentifierHandle Source);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPIdentifier_Clear(TElCMSOCSPIdentifierHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPIdentifier_get_ResponderID(TElCMSOCSPIdentifierHandle _Handle, TElOCSPResponderIDHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPIdentifier_get_Time(TElCMSOCSPIdentifierHandle _Handle, int64_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPIdentifier_set_Time(TElCMSOCSPIdentifierHandle _Handle, int64_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPIdentifier_Create(TElCMSOCSPIdentifierHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSOCSPIDENTIFIER */

#ifdef SB_USE_CLASS_TELCMSOCSPRESPONSESID
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPResponsesID_Clear(TElCMSOCSPResponsesIDHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPResponsesID_Import(TElCMSOCSPResponsesIDHandle _Handle, TElOCSPResponseHandle Response, int32_t HashAlgorithm);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPResponsesID_OCSPResponseMatches(TElCMSOCSPResponsesIDHandle _Handle, TElOCSPResponseHandle Response, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPResponsesID_get_OCSPIdentifier(TElCMSOCSPResponsesIDHandle _Handle, TElCMSOCSPIdentifierHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPResponsesID_get_OCSPHash(TElCMSOCSPResponsesIDHandle _Handle, TElCMSHashHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSOCSPResponsesID_Create(TElCMSOCSPResponsesIDHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSOCSPRESPONSESID */

#ifdef SB_USE_CLASS_TELCMSREVOCATIONREF
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_AddCRLID(TElCMSRevocationRefHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_AddOCSPID(TElCMSRevocationRefHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_RemoveCRLID(TElCMSRevocationRefHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_RemoveOCSPID(TElCMSRevocationRefHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_ClearCRLIDs(TElCMSRevocationRefHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_ClearOCSPIDs(TElCMSRevocationRefHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_Import(TElCMSRevocationRefHandle _Handle, TListHandle CRLList, TListHandle OCSPList);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_Import2(TElCMSRevocationRefHandle _Handle, TElMemoryCRLStorageHandle CRLs, TListHandle OCSPList);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_get_CRLIDs(TElCMSRevocationRefHandle _Handle, int32_t Index, TElCMSCRLValidatedIDHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_get_OCSPIDs(TElCMSRevocationRefHandle _Handle, int32_t Index, TElCMSOCSPResponsesIDHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_get_CRLIDCount(TElCMSRevocationRefHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_get_OCSPIDCount(TElCMSRevocationRefHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_get_OtherRevType(TElCMSRevocationRefHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_set_OtherRevType(TElCMSRevocationRefHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_get_OtherRevRefs(TElCMSRevocationRefHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_set_OtherRevRefs(TElCMSRevocationRefHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRef_Create(TElCMSRevocationRefHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSREVOCATIONREF */

#ifdef SB_USE_CLASS_TELCMSREVOCATIONREFS
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_Add(TElCMSRevocationRefsHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_Remove(TElCMSRevocationRefsHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_Clear(TElCMSRevocationRefsHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_get_Refs(TElCMSRevocationRefsHandle _Handle, int32_t Index, TElCMSRevocationRefHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_get_RefCount(TElCMSRevocationRefsHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_get_AddASN1ExplicitTagsForEmptyRecords(TElCMSRevocationRefsHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_set_AddASN1ExplicitTagsForEmptyRecords(TElCMSRevocationRefsHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_get_WriteMode(TElCMSRevocationRefsHandle _Handle, TSBCMSRevocationRefsWriteModeRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_set_WriteMode(TElCMSRevocationRefsHandle _Handle, TSBCMSRevocationRefsWriteModeRaw Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationRefs_Create(TElCMSRevocationRefsHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSREVOCATIONREFS */

#ifdef SB_USE_CLASS_TELCMSREVOCATIONVALUES
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_AddCRL(TElCMSRevocationValuesHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_AddOCSPResponse(TElCMSRevocationValuesHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_RemoveCRL(TElCMSRevocationValuesHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_RemoveOCSPResponse(TElCMSRevocationValuesHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_ClearCRLs(TElCMSRevocationValuesHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_ClearOCSPResponses(TElCMSRevocationValuesHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_Clear(TElCMSRevocationValuesHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_get_CRLs(TElCMSRevocationValuesHandle _Handle, int32_t Index, TElCertificateRevocationListHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_get_OCSPResponses(TElCMSRevocationValuesHandle _Handle, int32_t Index, TElOCSPResponseHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_get_CRLCount(TElCMSRevocationValuesHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_get_OCSPResponseCount(TElCMSRevocationValuesHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_get_OtherRevType(TElCMSRevocationValuesHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_set_OtherRevType(TElCMSRevocationValuesHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_get_OtherRevValues(TElCMSRevocationValuesHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_set_OtherRevValues(TElCMSRevocationValuesHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSRevocationValues_Create(TElCMSRevocationValuesHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSREVOCATIONVALUES */

#ifdef SB_USE_CLASS_TELCMSCONTENT
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_Clear(TElCMSContentHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_Init(TElCMSContentHandle _Handle, TStreamHandle Stream, int64_t Offset, int64_t Size);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_Init_1(TElCMSContentHandle _Handle, void * Buffer, int32_t Size);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_Read(TElCMSContentHandle _Handle, void * Buffer, int32_t Size, int64_t Offset, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_AddKnownHash(TElCMSContentHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_AddKnownHash_1(TElCMSContentHandle _Handle, int32_t AlgID, const uint8_t pHashAlgParams[], int32_t szHashAlgParams, const uint8_t pHash[], int32_t szHash, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_RemoveKnownHash(TElCMSContentHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_ClearKnownHashes(TElCMSContentHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_get_Size(TElCMSContentHandle _Handle, int64_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_get_ContentIsHash(TElCMSContentHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_set_ContentIsHash(TElCMSContentHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_get_RawContent(TElCMSContentHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_get_KnownHashes(TElCMSContentHandle _Handle, int32_t Index, TElCMSHashHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_get_KnownHashCount(TElCMSContentHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_Create(TElCMSContentHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_Create_1(TElPKCS7SignedDataHandle Data, TElCMSContentHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContent_Create_2(TElPKCS7SignedAndEnvelopedDataHandle Data, TElCMSContentHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSCONTENT */

#ifdef SB_USE_CLASS_TELCMSSIGPOLICYQUALIFIER
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_Clear(TElCMSSigPolicyQualifierHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_AddNoticeNumber(TElCMSSigPolicyQualifierHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_RemoveNoticeNumber(TElCMSSigPolicyQualifierHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_ClearNoticeNumbers(TElCMSSigPolicyQualifierHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_get_ID(TElCMSSigPolicyQualifierHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_set_ID(TElCMSSigPolicyQualifierHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_get_QualifierType(TElCMSSigPolicyQualifierHandle _Handle, TSBSigPolicyQualifierTypeRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_set_QualifierType(TElCMSSigPolicyQualifierHandle _Handle, TSBSigPolicyQualifierTypeRaw Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_get_URI(TElCMSSigPolicyQualifierHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_set_URI(TElCMSSigPolicyQualifierHandle _Handle, const char * pcValue, int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_get_ExplicitText(TElCMSSigPolicyQualifierHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_set_ExplicitText(TElCMSSigPolicyQualifierHandle _Handle, const char * pcValue, int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_get_Organization(TElCMSSigPolicyQualifierHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_set_Organization(TElCMSSigPolicyQualifierHandle _Handle, const char * pcValue, int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_get_CustomData(TElCMSSigPolicyQualifierHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_set_CustomData(TElCMSSigPolicyQualifierHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_get_NoticeNumbers(TElCMSSigPolicyQualifierHandle _Handle, int32_t Index, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_set_NoticeNumbers(TElCMSSigPolicyQualifierHandle _Handle, int32_t Index, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_get_NoticeNumberCount(TElCMSSigPolicyQualifierHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_get_UseUTF8(TElCMSSigPolicyQualifierHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_set_UseUTF8(TElCMSSigPolicyQualifierHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigPolicyQualifier_Create(TElCMSSigPolicyQualifierHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSSIGPOLICYQUALIFIER */

#ifdef SB_USE_CLASS_TELCMSSIGNATUREPOLICY
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_AddQualifier(TElCMSSignaturePolicyHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_RemoveQualifier(TElCMSSignaturePolicyHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_ClearQualifiers(TElCMSSignaturePolicyHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_Clear(TElCMSSignaturePolicyHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_get_PolicyIdentifier(TElCMSSignaturePolicyHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_set_PolicyIdentifier(TElCMSSignaturePolicyHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_get_PolicyHash(TElCMSSignaturePolicyHandle _Handle, TElCMSHashHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_get_Qualifiers(TElCMSSignaturePolicyHandle _Handle, int32_t Index, TElCMSSigPolicyQualifierHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_get_QualifierCount(TElCMSSignaturePolicyHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_get_Implied(TElCMSSignaturePolicyHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_set_Implied(TElCMSSignaturePolicyHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicy_Create(TElCMSSignaturePolicyHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSSIGNATUREPOLICY */

#ifdef SB_USE_CLASS_TELCMSCONTENTHINTS
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentHints_get_ContentDescription(TElCMSContentHintsHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentHints_set_ContentDescription(TElCMSContentHintsHandle _Handle, const char * pcValue, int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentHints_get_ContentType(TElCMSContentHintsHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentHints_set_ContentType(TElCMSContentHintsHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentHints_Create(TElCMSContentHintsHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSCONTENTHINTS */

#ifdef SB_USE_CLASS_TELCMSCONTENTREFERENCE
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentReference_get_ContentType(TElCMSContentReferenceHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentReference_set_ContentType(TElCMSContentReferenceHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentReference_get_ContentIdentifier(TElCMSContentReferenceHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentReference_set_ContentIdentifier(TElCMSContentReferenceHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentReference_get_OriginatorSignatureValue(TElCMSContentReferenceHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentReference_set_OriginatorSignatureValue(TElCMSContentReferenceHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentReference_Create(TElCMSContentReferenceHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSCONTENTREFERENCE */

#ifdef SB_USE_CLASS_TELCMSCOMMITMENTTYPEINDICATION
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_Clear(TElCMSCommitmentTypeIndicationHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_get_ProofOfOrigin(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_set_ProofOfOrigin(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_get_ProofOfReceipt(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_set_ProofOfReceipt(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_get_ProofOfDelivery(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_set_ProofOfDelivery(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_get_ProofOfSender(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_set_ProofOfSender(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_get_ProofOfApproval(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_set_ProofOfApproval(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_get_ProofOfCreation(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_set_ProofOfCreation(TElCMSCommitmentTypeIndicationHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_get_CustomIdentifiers(TElCMSCommitmentTypeIndicationHandle _Handle, TElByteArrayListHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSCommitmentTypeIndication_Create(TElCMSCommitmentTypeIndicationHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSCOMMITMENTTYPEINDICATION */

#ifdef SB_USE_CLASS_TELCMSSIGNERLOCATION
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerLocation_Clear(TElCMSSignerLocationHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerLocation_AddPostalAddressLine(TElCMSSignerLocationHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerLocation_RemovePostalAddressLine(TElCMSSignerLocationHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerLocation_ClearPostalAddressLines(TElCMSSignerLocationHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerLocation_get_CountryName(TElCMSSignerLocationHandle _Handle, TElASN1DirectoryStringHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerLocation_get_LocalityName(TElCMSSignerLocationHandle _Handle, TElASN1DirectoryStringHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerLocation_get_PostalAddressLines(TElCMSSignerLocationHandle _Handle, int32_t Index, TElASN1DirectoryStringHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerLocation_get_PostalAddressLineCount(TElCMSSignerLocationHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerLocation_Create(TElCMSSignerLocationHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSSIGNERLOCATION */

#ifdef SB_USE_CLASS_TELCMSSIGNINGCERTIFICATE
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_Clear(TElCMSSigningCertificateHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_AddCertID(TElCMSSigningCertificateHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_RemoveCertID(TElCMSSigningCertificateHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_ClearCertIDs(TElCMSSigningCertificateHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_Corresponds(TElCMSSigningCertificateHandle _Handle, TElX509CertificateHandle Cert, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_get_CertIDs(TElCMSSigningCertificateHandle _Handle, int32_t Index, TElCMSSignerIdentifierHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_get_CertIDCount(TElCMSSigningCertificateHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_get_SigningCertificateType(TElCMSSigningCertificateHandle _Handle, TSBCMSSigningCertificateTypeRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_set_SigningCertificateType(TElCMSSigningCertificateHandle _Handle, TSBCMSSigningCertificateTypeRaw Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSigningCertificate_Create(TElCMSSigningCertificateHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSSIGNINGCERTIFICATE */

#ifdef SB_USE_CLASS_TELCMSSIGNERATTRIBUTES
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_Clear(TElCMSSignerAttributesHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_get_ClaimedAttributes(TElCMSSignerAttributesHandle _Handle, TElPKCS7AttributesHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_get_CertifiedAttributesBinary(TElCMSSignerAttributesHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_set_CertifiedAttributesBinary(TElCMSSignerAttributesHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_get_SignedAssertions(TElCMSSignerAttributesHandle _Handle, TElPKCS7AttributesHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_get_Version(TElCMSSignerAttributesHandle _Handle, TSBCMSSignerAttributesVersionRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_set_Version(TElCMSSignerAttributesHandle _Handle, TSBCMSSignerAttributesVersionRaw Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_get_AttributeCertificateType(TElCMSSignerAttributesHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_set_AttributeCertificateType(TElCMSSignerAttributesHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignerAttributes_Create(TElCMSSignerAttributesHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSSIGNERATTRIBUTES */

#ifdef SB_USE_CLASS_TELCMSSIGNATUREPOLICYSTORE
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_Clear(TElCMSSignaturePolicyStoreHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_get_DocSpecOID(TElCMSSignaturePolicyStoreHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_set_DocSpecOID(TElCMSSignaturePolicyStoreHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_get_DocSpecURI(TElCMSSignaturePolicyStoreHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_set_DocSpecURI(TElCMSSignaturePolicyStoreHandle _Handle, const char * pcValue, int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_get_EncodedPolicyDocument(TElCMSSignaturePolicyStoreHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_set_EncodedPolicyDocument(TElCMSSignaturePolicyStoreHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_get_PolicyLocalURI(TElCMSSignaturePolicyStoreHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_set_PolicyLocalURI(TElCMSSignaturePolicyStoreHandle _Handle, const char * pcValue, int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_get_PolicyStoreType(TElCMSSignaturePolicyStoreHandle _Handle, TSBCMSSignaturePolicyStoreTypeRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_set_PolicyStoreType(TElCMSSignaturePolicyStoreHandle _Handle, TSBCMSSignaturePolicyStoreTypeRaw Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignaturePolicyStore_Create(TElCMSSignaturePolicyStoreHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSSIGNATUREPOLICYSTORE */

#ifdef SB_USE_CLASS_TELCMSSIGNATURE
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_Assign(TElCMSSignatureHandle _Handle, TElCMSSignatureHandle Source);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_AddCountersignature(TElCMSSignatureHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_AddCountersignature_1(TElCMSSignatureHandle _Handle, const uint8_t pSignature[], int32_t szSignature, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_AddTimestamp(TElCMSSignatureHandle _Handle, TElCustomTSPClientHandle TSPClient, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_AddTimestamp_1(TElCMSSignatureHandle _Handle, const uint8_t pTimestamp[], int32_t szTimestamp, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_AddContentTimestamp(TElCMSSignatureHandle _Handle, TElCustomTSPClientHandle TSPClient, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_AddContentTimestamp_1(TElCMSSignatureHandle _Handle, const uint8_t pTimestamp[], int32_t szTimestamp, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_AddValidationTimestamp(TElCMSSignatureHandle _Handle, TElCustomTSPClientHandle TSPClient, TSBCMSTimestampTypeRaw TimestampType, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_AddCompleteValidationData(TElCMSSignatureHandle _Handle, TElCustomCertStorageHandle Chain, TElMemoryCRLStorageHandle CRLs, TListHandle OCSPResponses, int8_t ReferencesOnly);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_AddCompleteValidationDataItem(TElCMSSignatureHandle _Handle, TElX509CertificateHandle Cert, TElMemoryCRLStorageHandle CRLs, TListHandle OCSPResponses, int8_t ReferencesOnly);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_RemoveCountersignature(TElCMSSignatureHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_RemoveTimestamp(TElCMSSignatureHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_RemoveContentTimestamp(TElCMSSignatureHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_RemoveValidationTimestamp(TElCMSSignatureHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_ClearCountersignatures(TElCMSSignatureHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_ClearTimestamps(TElCMSSignatureHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_ClearContentTimestamps(TElCMSSignatureHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_ClearValidationTimestamps(TElCMSSignatureHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_Sign(TElCMSSignatureHandle _Handle, TElX509CertificateHandle Cert, TElCustomCertStorageHandle Chain);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_SignRemote(TElCMSSignatureHandle _Handle, TElX509CertificateHandle Cert, TSBCMSRemoteSignCallback pMethodSignCallback, void * pDataSignCallback, void * Param, TElCustomCertStorageHandle Chain);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_SignRemote_1(TElCMSSignatureHandle _Handle, TElX509CertificateHandle Cert, TSBCMSRemoteSignCallback pMethodSignCallback, void * pDataSignCallback, int8_t ExternalHashCalculation, void * Param, TElCustomCertStorageHandle Chain);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_Validate(TElCMSSignatureHandle _Handle, TSBCMSSignatureValidityRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_Validate_1(TElCMSSignatureHandle _Handle, TSBCMSValidationOptionsRaw Options, TSBCMSAdvancedSignatureValidityRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_Validate_2(TElCMSSignatureHandle _Handle, TElX509CertificateValidatorHandle Validator, TSBCMSValidationOptionsRaw Options, TSBCMSAdvancedSignatureValidityRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_GetSignedCMSMessage(TElCMSSignatureHandle _Handle, TElSignedCMSMessageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_InitiateAsyncSign(TElCMSSignatureHandle _Handle, TElX509CertificateHandle Cert, TElCustomCertStorageHandle Chain, TElDCAsyncStateHandle * State);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_InitiateAsyncSign_1(TElCMSSignatureHandle _Handle, TElDCAsyncStateHandle * State);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_InitiateAsyncSign_2(TElCMSSignatureHandle _Handle, TSBDCAsyncSignMethodRaw AsyncSignMethod, TElDCAsyncStateHandle * State);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_CompleteAsyncSign(TElCMSSignatureHandle _Handle, TElDCAsyncStateHandle AsyncState);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_UpdateCMS(TElCMSSignatureHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_Content(TElCMSSignatureHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_Signer(TElCMSSignatureHandle _Handle, TElPKCS7IssuerHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_DigestAlgorithm(TElCMSSignatureHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_DigestAlgorithm(TElCMSSignatureHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_EncryptedDigest(TElCMSSignatureHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_PublicKeyAlgorithm(TElCMSSignatureHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_PublicKeyAlgorithm(TElCMSSignatureHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_FingerprintAlgorithm(TElCMSSignatureHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_FingerprintAlgorithm(TElCMSSignatureHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_CustomSignedAttributes(TElCMSSignatureHandle _Handle, TElPKCS7AttributesHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_CustomUnsignedAttributes(TElCMSSignatureHandle _Handle, TElPKCS7AttributesHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_SignedAttributesPresent(TElCMSSignatureHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_SignatureAlgorithm(TElCMSSignatureHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_Countersignatures(TElCMSSignatureHandle _Handle, int32_t Index, TElCMSSignatureHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_Timestamps(TElCMSSignatureHandle _Handle, int32_t Index, TElCMSTimestampHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_ContentTimestamps(TElCMSSignatureHandle _Handle, int32_t Index, TElCMSTimestampHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_ValidationTimestamps(TElCMSSignatureHandle _Handle, int32_t Index, TElCMSTimestampHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_CountersignatureCount(TElCMSSignatureHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_TimestampCount(TElCMSSignatureHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_ContentTimestampCount(TElCMSSignatureHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_LastValidationResult(TElCMSSignatureHandle _Handle, TSBCMSAdvancedSignatureValidityRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_LastValidatorValidity(TElCMSSignatureHandle _Handle, TSBCertificateValidityRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_LastValidatorReason(TElCMSSignatureHandle _Handle, TSBCertificateValidityReasonRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_ContentType(TElCMSSignatureHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_ContentType(TElCMSSignatureHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_SigningTime(TElCMSSignatureHandle _Handle, int64_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_SigningTime(TElCMSSignatureHandle _Handle, int64_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_SignaturePolicy(TElCMSSignatureHandle _Handle, TElCMSSignaturePolicyHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_ContentHints(TElCMSSignatureHandle _Handle, TElCMSContentHintsHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_ContentIdentifier(TElCMSSignatureHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_ContentIdentifier(TElCMSSignatureHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_ContentReference(TElCMSSignatureHandle _Handle, TElCMSContentReferenceHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_CommitmentTypeIndication(TElCMSSignatureHandle _Handle, TElCMSCommitmentTypeIndicationHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_CommitmentTypeIndication(TElCMSSignatureHandle _Handle, TElCMSCommitmentTypeIndicationHandle Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_SignerLocation(TElCMSSignatureHandle _Handle, TElCMSSignerLocationHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_SigningCertificate(TElCMSSignatureHandle _Handle, TElCMSSigningCertificateHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_MessageDigest(TElCMSSignatureHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_MimeType(TElCMSSignatureHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_MimeType(TElCMSSignatureHandle _Handle, const char * pcValue, int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_CertificateRefs(TElCMSSignatureHandle _Handle, TElCMSCertificateRefsHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_CertificateValues(TElCMSSignatureHandle _Handle, TElCustomCertStorageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_RevocationRefs(TElCMSSignatureHandle _Handle, TElCMSRevocationRefsHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_RevocationValues(TElCMSSignatureHandle _Handle, TElCMSRevocationValuesHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_SignerAttributes(TElCMSSignatureHandle _Handle, TElCMSSignerAttributesHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_SignaturePolicyStore(TElCMSSignatureHandle _Handle, TElCMSSignaturePolicyStoreHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_ValidationTimestampCount(TElCMSSignatureHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_State(TElCMSSignatureHandle _Handle, TSBCMSSignatureStateRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_SigningOptions(TElCMSSignatureHandle _Handle, TSBCMSSigningOptionsRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_SigningOptions(TElCMSSignatureHandle _Handle, TSBCMSSigningOptionsRaw Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_UsePSS(TElCMSSignatureHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_UsePSS(TElCMSSignatureHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_UseCachedDigests(TElCMSSignatureHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_UseCachedDigests(TElCMSSignatureHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_LiberalMode(TElCMSSignatureHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_LiberalMode(TElCMSSignatureHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_AuthenticodeMode(TElCMSSignatureHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_AuthenticodeMode(TElCMSSignatureHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_OnBeforeSign(TElCMSSignatureHandle _Handle, TSBCMSBeforeSignEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_OnBeforeSign(TElCMSSignatureHandle _Handle, TSBCMSBeforeSignEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_OnBeforeVerify(TElCMSSignatureHandle _Handle, TSBCMSBeforeVerifyEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_OnBeforeVerify(TElCMSSignatureHandle _Handle, TSBCMSBeforeVerifyEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_OnCertificateNeeded(TElCMSSignatureHandle _Handle, TSBCMSCertificateNeededEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_OnCertificateNeeded(TElCMSSignatureHandle _Handle, TSBCMSCertificateNeededEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_OnIsCertificateTrusted(TElCMSSignatureHandle _Handle, TSBCMSCertificateTrustedEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_OnIsCertificateTrusted(TElCMSSignatureHandle _Handle, TSBCMSCertificateTrustedEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_OnCRLNeeded(TElCMSSignatureHandle _Handle, TSBCMSCRLNeededEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_OnCRLNeeded(TElCMSSignatureHandle _Handle, TSBCMSCRLNeededEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_OnOCSPResponseNeeded(TElCMSSignatureHandle _Handle, TSBCMSOCSPResponseNeededEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_OnOCSPResponseNeeded(TElCMSSignatureHandle _Handle, TSBCMSOCSPResponseNeededEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_OnItemValidated(TElCMSSignatureHandle _Handle, TSBCMSItemValidatedEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_OnItemValidated(TElCMSSignatureHandle _Handle, TSBCMSItemValidatedEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_get_OnProgress(TElCMSSignatureHandle _Handle, TSBProgressEvent * pMethodOutResult, void * * pDataOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_set_OnProgress(TElCMSSignatureHandle _Handle, TSBProgressEvent pMethodValue, void * pDataValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_Create(TObjectHandle Subject, TElCMSSignatureHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_Create_1(TObjectHandle Subject, TElPKCS7SignerHandle PKCS7Signer, TElCMSSignatureHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSSignature_Create_2(TObjectHandle Subject, const uint8_t pData[], int32_t szData, TElCMSSignatureHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSSIGNATURE */

#ifdef SB_USE_CLASS_TELCMSTIMESTAMP
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_Validate(TElCMSTimestampHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_UpdateCMS(TElCMSTimestampHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_Certificates(TElCMSTimestampHandle _Handle, TElCustomCertStorageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_CRLs(TElCMSTimestampHandle _Handle, TElMemoryCRLStorageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_OCSPResponses(TElCMSTimestampHandle _Handle, TElOCSPResponseStorageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_Signatures(TElCMSTimestampHandle _Handle, int32_t Index, TElCMSSignatureHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_SignatureCount(TElCMSTimestampHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_Info(TElCMSTimestampHandle _Handle, TElClientTSPInfoHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_TimestampType(TElCMSTimestampHandle _Handle, TSBCMSTimestampTypeRaw * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_UseCachedDigests(TElCMSTimestampHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_set_UseCachedDigests(TElCMSTimestampHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_DigestCalcType(TElCMSTimestampHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_get_CMS(TElCMSTimestampHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_Create(TObjectHandle Subject, TElCMSTimestampHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSTimestamp_Create_1(TObjectHandle Subject, const uint8_t pData[], int32_t szData, TElCMSTimestampHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSTIMESTAMP */

#ifdef SB_USE_CLASS_TELATSHASHINDEXPROCESSOR
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_Clear(TElATSHashIndexProcessorHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_Load(TElATSHashIndexProcessorHandle _Handle, const uint8_t pBuffer[], int32_t szBuffer, int32_t StartIndex, int32_t Count);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_CertPresent(TElATSHashIndexProcessorHandle _Handle, TElX509CertificateHandle Cert, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_CRLPresent(TElATSHashIndexProcessorHandle _Handle, TElCertificateRevocationListHandle Crl, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_OCSPResponsePresent(TElATSHashIndexProcessorHandle _Handle, TElOCSPResponseHandle OcspResponse, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_get_CertHashes(TElATSHashIndexProcessorHandle _Handle, int32_t Index, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_get_CertHashesCount(TElATSHashIndexProcessorHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_get_RevInfoHashes(TElATSHashIndexProcessorHandle _Handle, int32_t Index, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_get_RevInfoHashesCount(TElATSHashIndexProcessorHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_get_AttributeHashes(TElATSHashIndexProcessorHandle _Handle, int32_t Index, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_get_AttributeHashesCount(TElATSHashIndexProcessorHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_get_HashAlgorithm(TElATSHashIndexProcessorHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElATSHashIndexProcessor_Create(TElATSHashIndexProcessorHandle * OutResult);
#endif /* SB_USE_CLASS_TELATSHASHINDEXPROCESSOR */

#ifdef SB_USE_CLASS_TELCMSMESSAGE
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_Clear(TElCMSMessageHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_CreateNew(TElCMSMessageHandle _Handle, TStreamHandle DataStream, int64_t Offset, int64_t Size);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_CreateNew_1(TElCMSMessageHandle _Handle, void * Buffer, int32_t Size);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_Open(TElCMSMessageHandle _Handle, TStreamHandle CMSStream, TStreamHandle DataStream, int64_t CMSCount, int64_t DataCount);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_Save(TElCMSMessageHandle _Handle, TStreamHandle DestStream);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_Close(TElCMSMessageHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_get_Content(TElCMSMessageHandle _Handle, TElCMSContentHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_get_Detached(TElCMSMessageHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_set_Detached(TElCMSMessageHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_get_UseUndefSize(TElCMSMessageHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_set_UseUndefSize(TElCMSMessageHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElCMSMessage_Create(TComponentHandle AOwner, TElCMSMessageHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSMESSAGE */

#ifdef SB_USE_CLASS_TELCMSCONTENTHASH
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentHash_Clear(TElCMSContentHashHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentHash_GetHash(TElCMSContentHashHandle _Handle, int32_t Alg, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentHash_SetHash(TElCMSContentHashHandle _Handle, int32_t Alg, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElCMSContentHash_Create(TElCMSContentHashHandle * OutResult);
#endif /* SB_USE_CLASS_TELCMSCONTENTHASH */

#ifdef SB_USE_CLASS_TELSIGNEDCMSMESSAGE
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_AddSignature(TElSignedCMSMessageHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_RemoveSignature(TElSignedCMSMessageHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_ClearSignatures(TElSignedCMSMessageHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_ExtractAllCertificates(TElSignedCMSMessageHandle _Handle, TElCustomCertStorageHandle Storage);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_ExtractAllValidationElements(TElSignedCMSMessageHandle _Handle, TElCustomCertStorageHandle Storage, TElMemoryCRLStorageHandle Crls, TListHandle Ocsps);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_ContentType(TElSignedCMSMessageHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_set_ContentType(TElSignedCMSMessageHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_ContentVersion(TElSignedCMSMessageHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_set_ContentVersion(TElSignedCMSMessageHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_Certificates(TElSignedCMSMessageHandle _Handle, TElCustomCertStorageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_CRLs(TElSignedCMSMessageHandle _Handle, TElMemoryCRLStorageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_OCSPResponses(TElSignedCMSMessageHandle _Handle, TElOCSPResponseStorageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_Signatures(TElSignedCMSMessageHandle _Handle, int32_t Index, TElCMSSignatureHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_SignatureCount(TElSignedCMSMessageHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_UseImplicitContent(TElSignedCMSMessageHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_set_UseImplicitContent(TElSignedCMSMessageHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_SerializeContentOnUpdate(TElSignedCMSMessageHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_set_SerializeContentOnUpdate(TElSignedCMSMessageHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_get_CertStorage(TElSignedCMSMessageHandle _Handle, TElCustomCertStorageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_set_CertStorage(TElSignedCMSMessageHandle _Handle, TElCustomCertStorageHandle Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedCMSMessage_Create(TComponentHandle AOwner, TElSignedCMSMessageHandle * OutResult);
#endif /* SB_USE_CLASS_TELSIGNEDCMSMESSAGE */

#ifdef SB_USE_CLASS_TELSIGNEDANDENVELOPEDCMSMESSAGE
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_get_CryptoProviderManager(TElSignedAndEnvelopedCMSMessageHandle _Handle, TElCustomCryptoProviderManagerHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_set_CryptoProviderManager(TElSignedAndEnvelopedCMSMessageHandle _Handle, TElCustomCryptoProviderManagerHandle Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_get_Recipients(TElSignedAndEnvelopedCMSMessageHandle _Handle, TElCustomCertStorageHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_set_Recipients(TElSignedAndEnvelopedCMSMessageHandle _Handle, TElCustomCertStorageHandle Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_get_Algorithm(TElSignedAndEnvelopedCMSMessageHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_set_Algorithm(TElSignedAndEnvelopedCMSMessageHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_get_BitsInKey(TElSignedAndEnvelopedCMSMessageHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_set_BitsInKey(TElSignedAndEnvelopedCMSMessageHandle _Handle, int32_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_get_UseImplicitContentEncoding(TElSignedAndEnvelopedCMSMessageHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_set_UseImplicitContentEncoding(TElSignedAndEnvelopedCMSMessageHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_get_UseOAEP(TElSignedAndEnvelopedCMSMessageHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_set_UseOAEP(TElSignedAndEnvelopedCMSMessageHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_get_AlignEncryptedKey(TElSignedAndEnvelopedCMSMessageHandle _Handle, int8_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_set_AlignEncryptedKey(TElSignedAndEnvelopedCMSMessageHandle _Handle, int8_t Value);
SB_IMPORT uint32_t SB_APIENTRY TElSignedAndEnvelopedCMSMessage_Create(TComponentHandle AOwner, TElSignedAndEnvelopedCMSMessageHandle * OutResult);
#endif /* SB_USE_CLASS_TELSIGNEDANDENVELOPEDCMSMESSAGE */

#ifdef __cplusplus
};	/* extern "C" */
#endif

extern zend_class_entry *TElASN1DirectoryString_ce_ptr;
extern zend_class_entry *TElCMSProperty_ce_ptr;
extern zend_class_entry *TElCMSHash_ce_ptr;
extern zend_class_entry *TElCMSSignerIdentifier_ce_ptr;
extern zend_class_entry *TElCMSCertificateRefs_ce_ptr;
extern zend_class_entry *TElCMSCRLIdentifier_ce_ptr;
extern zend_class_entry *TElCMSCRLValidatedID_ce_ptr;
extern zend_class_entry *TElCMSOCSPIdentifier_ce_ptr;
extern zend_class_entry *TElCMSOCSPResponsesID_ce_ptr;
extern zend_class_entry *TElCMSRevocationRef_ce_ptr;
extern zend_class_entry *TElCMSRevocationRefs_ce_ptr;
extern zend_class_entry *TElCMSRevocationValues_ce_ptr;
extern zend_class_entry *TElCMSContent_ce_ptr;
extern zend_class_entry *TElCMSSigPolicyQualifier_ce_ptr;
extern zend_class_entry *TElCMSSignaturePolicy_ce_ptr;
extern zend_class_entry *TElCMSContentHints_ce_ptr;
extern zend_class_entry *TElCMSContentReference_ce_ptr;
extern zend_class_entry *TElCMSCommitmentTypeIndication_ce_ptr;
extern zend_class_entry *TElCMSSignerLocation_ce_ptr;
extern zend_class_entry *TElCMSSigningCertificate_ce_ptr;
extern zend_class_entry *TElCMSSignerAttributes_ce_ptr;
extern zend_class_entry *TElCMSSignaturePolicyStore_ce_ptr;
extern zend_class_entry *TElCMSSignature_ce_ptr;
extern zend_class_entry *TElCMSTimestamp_ce_ptr;
extern zend_class_entry *TElATSHashIndexProcessor_ce_ptr;
extern zend_class_entry *TElCMSMessage_ce_ptr;
extern zend_class_entry *TElCMSContentHash_ce_ptr;
extern zend_class_entry *TElSignedCMSMessage_ce_ptr;
extern zend_class_entry *TElSignedAndEnvelopedCMSMessage_ce_ptr;

void SB_CALLBACK TSBCMSBeforeSignEventRaw(void * _ObjectData, TObjectHandle Sender, TElPublicKeyCryptoHandle Crypto);
void SB_CALLBACK TSBCMSBeforeVerifyEventRaw(void * _ObjectData, TObjectHandle Sender, TElPublicKeyCryptoHandle Crypto);
void SB_CALLBACK TSBCMSCertificateTrustedEventRaw(void * _ObjectData, TObjectHandle Sender, TElX509CertificateHandle Cert, int8_t * Trusted);
void SB_CALLBACK TSBCMSItemValidatedEventRaw(void * _ObjectData, TObjectHandle Sender, TSBCMSItemTypeRaw ItemType, TObjectHandle Item, int8_t * Valid);
void SB_CALLBACK TSBCMSCRLNeededEventRaw(void * _ObjectData, TObjectHandle Sender, TElCMSCRLValidatedIDHandle ID, TElX509CertificateHandle Cert, TElCertificateRevocationListHandle * Crl);
void SB_CALLBACK TSBCMSOCSPResponseNeededEventRaw(void * _ObjectData, TObjectHandle Sender, TElCMSOCSPResponsesIDHandle ID, TElX509CertificateHandle Cert, TElOCSPResponseHandle * Ocsp);
void SB_CALLBACK TSBCMSRemoteSignCallbackRaw(void * _ObjectData, TObjectHandle Sender, TObjectHandle Param, const uint8_t pHash[], int32_t szHash, uint8_t pSignedHash[], int32_t * szSignedHash, int8_t * OutResult);
void Register_TElASN1DirectoryString(TSRMLS_D);
void Register_TElCMSProperty(TSRMLS_D);
void Register_TElCMSHash(TSRMLS_D);
void Register_TElCMSSignerIdentifier(TSRMLS_D);
void Register_TElCMSCertificateRefs(TSRMLS_D);
void Register_TElCMSCRLIdentifier(TSRMLS_D);
void Register_TElCMSCRLValidatedID(TSRMLS_D);
void Register_TElCMSOCSPIdentifier(TSRMLS_D);
void Register_TElCMSOCSPResponsesID(TSRMLS_D);
void Register_TElCMSRevocationRef(TSRMLS_D);
void Register_TElCMSRevocationRefs(TSRMLS_D);
void Register_TElCMSRevocationValues(TSRMLS_D);
void Register_TElCMSContent(TSRMLS_D);
void Register_TElCMSSigPolicyQualifier(TSRMLS_D);
void Register_TElCMSSignaturePolicy(TSRMLS_D);
void Register_TElCMSContentHints(TSRMLS_D);
void Register_TElCMSContentReference(TSRMLS_D);
void Register_TElCMSCommitmentTypeIndication(TSRMLS_D);
void Register_TElCMSSignerLocation(TSRMLS_D);
void Register_TElCMSSigningCertificate(TSRMLS_D);
void Register_TElCMSSignerAttributes(TSRMLS_D);
void Register_TElCMSSignaturePolicyStore(TSRMLS_D);
void Register_TElCMSSignature(TSRMLS_D);
void Register_TElCMSTimestamp(TSRMLS_D);
void Register_TElATSHashIndexProcessor(TSRMLS_D);
void Register_TElCMSMessage(TSRMLS_D);
void Register_TElCMSContentHash(TSRMLS_D);
void Register_TElSignedCMSMessage(TSRMLS_D);
void Register_TElSignedAndEnvelopedCMSMessage(TSRMLS_D);
SB_PHP_FUNCTION(SBCMS, GetTimestampTypeOID);
void Register_SBCMS_Constants(int module_number TSRMLS_DC);
void Register_SBCMS_Enum_Flags(TSRMLS_D);

#ifdef __cplusplus
extern "C" {
#endif

#ifdef SB_USE_GLOBAL_PROCS_CMS
SB_IMPORT uint32_t SB_APIENTRY SBCMS_GetTimestampTypeOID(TSBCMSTimestampTypeRaw TimestampType, uint8_t pOutResult[], int32_t * szOutResult);
#endif /* SB_USE_GLOBAL_PROCS_CMS */

#ifdef __cplusplus
};	/* extern "C" */
#endif

#ifdef __cplusplus
};	/* namespace SecureBlackbox */
#endif

#pragma pack(pop)

#endif  /* __INC_SBCMS */

