#ifndef __INC_SBPGPCONSTANTS
#define __INC_SBPGPCONSTANTS

#if _MSC_VER > 1000
#  pragma once
#endif // _MSC_VER > 1000

#include "sbdefs.h"
#include "sbphpcore.h"
#include "sbsystem.h"
#include "sbtypes.h"
#include "sbstrutils.h"
#include "sbconstants.h"
#include "sbutils.h"

#pragma pack(push, 1)

#ifdef __cplusplus
namespace SecureBlackbox {
#endif

#ifdef __cplusplus
extern "C" {
#endif
#define SB_PGP_ALGORITHM_PK_RSA 	1
#define SB_PGP_ALGORITHM_PK_RSA_ENCRYPT 	2
#define SB_PGP_ALGORITHM_PK_RSA_SIGN 	3
#define SB_PGP_ALGORITHM_PK_ELGAMAL_ENCRYPT 	16
#define SB_PGP_ALGORITHM_PK_DSA 	17
#define SB_PGP_ALGORITHM_PK_ECDH 	18
#define SB_PGP_ALGORITHM_PK_ECDSA 	19
#define SB_PGP_ALGORITHM_PK_ELGAMAL 	20
#define SB_PGP_ALGORITHM_X509 	100
#define SB_PGP_ALGORITHM_SK_PLAINTEXT 	0
#define SB_PGP_ALGORITHM_SK_IDEA 	1
#define SB_PGP_ALGORITHM_SK_3DES 	2
#define SB_PGP_ALGORITHM_SK_CAST5 	3
#define SB_PGP_ALGORITHM_SK_BLOWFISH 	4
#define SB_PGP_ALGORITHM_SK_AES128 	7
#define SB_PGP_ALGORITHM_SK_AES192 	8
#define SB_PGP_ALGORITHM_SK_AES256 	9
#define SB_PGP_ALGORITHM_SK_TWOFISH256 	10
#define SB_PGP_ALGORITHM_CM_UNCOMPRESSED 	0
#define SB_PGP_ALGORITHM_CM_ZIP 	1
#define SB_PGP_ALGORITHM_CM_ZLIB 	2
#define SB_PGP_ALGORITHM_CM_BZIP2 	3
#define SB_PGP_ALGORITHM_MD_AUTO 	0
#define SB_PGP_ALGORITHM_MD_MD5 	1
#define SB_PGP_ALGORITHM_MD_SHA1 	2
#define SB_PGP_ALGORITHM_MD_RIPEMD160 	3
#define SB_PGP_ALGORITHM_UNKNOWN 	-1
#define SB_PGP_ALGORITHM_MD_SHA256 	8
#define SB_PGP_ALGORITHM_MD_SHA384 	9
#define SB_PGP_ALGORITHM_MD_SHA512 	10
#define SB_PGP_ALGORITHM_MD_SHA224 	11
#define SB_PGP_MAXPUBNUMS 	6
#define SB_PGP_MAXSIGNUMS 	2
#define SB_PGP_CURVE_UNKNOWN 	0
#define SB_PGP_CURVE_P256 	1
#define SB_PGP_CURVE_P384 	2
#define SB_PGP_CURVE_P521 	3
#define SB_PGP_PKT_RESERVED 	0
#define SB_PGP_PKT_PUBKEYESK 	1
#define SB_PGP_PKT_SIGNATURE 	2
#define SB_PGP_PKT_SYMKEYESK 	3
#define SB_PGP_PKT_ONEPASSSIG 	4
#define SB_PGP_PKT_SECKEY 	5
#define SB_PGP_PKT_PUBKEY 	6
#define SB_PGP_PKT_SECSUBKEY 	7
#define SB_PGP_PKT_COMPRESSED 	8
#define SB_PGP_PKT_SYMENC 	9
#define SB_PGP_PKT_MARKER 	10
#define SB_PGP_PKT_LITERAL 	11
#define SB_PGP_PKT_TRUST 	12
#define SB_PGP_PKT_USERID 	13
#define SB_PGP_PKT_PUBSUBKEY 	14
#define SB_PGP_PKT_USERATTR 	17
#define SB_PGP_PKT_SYMENCPRO 	18
#define SB_PGP_PKT_MDC 	19
#define SB_PGP_SIGSUBPKT_CREATIONTIME 	2
#define SB_PGP_SIGSUBPKT_EXPIRATIONTIME 	3
#define SB_PGP_SIGSUBPKT_EXPORTABLE 	4
#define SB_PGP_SIGSUBPKT_TRUST 	5
#define SB_PGP_SIGSUBPKT_REGEXP 	6
#define SB_PGP_SIGSUBPKT_REVOCABLE 	7
#define SB_PGP_SIGSUBPKT_KEYEXPIRATIONTIME 	9
#define SB_PGP_SIGSUBPKT_PLACEHOLDER 	10
#define SB_PGP_SIGSUBPKT_PREFERREDSYMALG 	11
#define SB_PGP_SIGSUBPKT_REVOCATIONKEY 	12
#define SB_PGP_SIGSUBPKT_ISSUERKEYID 	16
#define SB_PGP_SIGSUBPKT_NOTATIONDATA 	20
#define SB_PGP_SIGSUBPKT_PREFERREDHASHALG 	21
#define SB_PGP_SIGSUBPKT_PREFERREDCOMPRALG 	22
#define SB_PGP_SIGSUBPKT_KEYSERVERPREF 	23
#define SB_PGP_SIGSUBPKT_PREFERREDKEYSERVER 	24
#define SB_PGP_SIGSUBPKT_PRIMARYUSERID 	25
#define SB_PGP_SIGSUBPKT_POLICYURL 	26
#define SB_PGP_SIGSUBPKT_KEYFLAGS 	27
#define SB_PGP_SIGSUBPKT_SIGNERSUSERID 	28
#define SB_PGP_SIGSUBPKT_REASONFORREVOCATION 	29
#define SB_PGP_SIGSUBPKT_FEATURES 	30
#define SB_PGP_SIGSUBPKT_TARGET 	31
#define SB_PGP_SIGSUBPKT_EMBEDDED_SIG 	32
#define SB_PGP_SIGSUBPKT_X509 	100
#define SB_PGP_SIGTYPE_DOCUMENT 	0
#define SB_PGP_SIGTYPE_TEXTDOCUMENT 	1
#define SB_PGP_SIGTYPE_STANDALONE 	2
#define SB_PGP_SIGTYPE_UID_GENERIC 	16
#define SB_PGP_SIGTYPE_UID_PERSONA 	17
#define SB_PGP_SIGTYPE_UID_CASUAL 	18
#define SB_PGP_SIGTYPE_UID_POSITIVE 	19
#define SB_PGP_SIGTYPE_SUBKEYBINDING 	24
#define SB_PGP_SIGTYPE_PKEYBINDING 	25
#define SB_PGP_SIGTYPE_DIRECTKEY 	31
#define SB_PGP_SIGTYPE_KEYREVOCATION 	32
#define SB_PGP_SIGTYPE_SUBKEYREVOCATION 	40
#define SB_PGP_SIGTYPE_CERTREVOCATION 	48
#define SB_PGP_SIGTYPE_TIMESTAMP 	64
#define SB_PGP_SIGTYPE_THIRDPARTY 	80
#define SB_PGP_SIGTYPE_NOTSPECIFIED 	255
#define SB_PGP_USERATTR_SUBPKT_IMAGE 	1
#define SB_PGP_USERATTR_IMAGE_JPEG 	1
#define SB_PGP_TRUST_BUCKSTOP 	128
#define SB_PGP_TRUST_CHECKED 	64
#define SB_PGP_TRUST_CONTIGUOUS 	32
#define SB_PGP_TRUST_DISABLED 	32
#define SB_PGP_TRUST_WARNONLY 	128
#define SB_PGP_KEYTRUST_UNDEFINED 	0
#define SB_PGP_KEYTRUST_UNKNOWN 	1
#define SB_PGP_KEYTRUST_NOTTRUST 	2
#define SB_PGP_KEYTRUST_USUALTRUST 	5
#define SB_PGP_KEYTRUST_ALWAYSTRUST 	6
#define SB_PGP_KEYTRUST_INKEYRING 	7
#define SB_PGP_SIGTRUST_UNDEFINED 	0
#define SB_PGP_SIGTRUST_UNKNOWN 	1
#define SB_PGP_SIGTRUST_NOTTRUST 	2
#define SB_PGP_SIGTRUST_REASONTRUST 	5
#define SB_PGP_SIGTRUST_COMPLETETRUST 	6
#define SB_PGP_SIGTRUST_ULTIMATETRUST 	7
#define SB_PGP_UIDTRUST_UNDEFINED 	0
#define SB_PGP_UIDTRUST_NOTTRUST 	1
#define SB_PGP_UIDTRUST_MARGINALTRUST 	2
#define SB_PGP_UIDTRUST_COMPLETETRUST 	3
#define SB_PGP_REVOCATION_NOREASON 	0
#define SB_PGP_REVOCATION_KEY_SUPERCEDED 	1
#define SB_PGP_REVOCATION_KEY_COMPROMISED 	2
#define SB_PGP_REVOCATION_KEY_RETIRED 	3
#define SB_PGP_REVOCATION_UID_NOLONGERVALID 	32
#define SB_PGP_ENCODING_AUTO 	0
#define SB_PGP_ENCODING_BINARY 	1
#define SB_PGP_ENCODING_UTF8 	2
#define SB_PGP_ENCODING_UTF16BE 	3
#define SB_PGP_ENCODING_UTF16LE 	4
#define SB_PGP_ENCODING_UTF32BE 	5
#define SB_PGP_ENCODING_UTF32LE 	6
#define SB_PGP_ENCODING_UNKNOWN 	100
#define SB_PGP_WRITEFLG_NOHEADER 	1
#define SB_PGP_WRITEFLG_SIGHASHHEADER 	3
#define SB_PGP_WRITEFLG_SIGONLYHASHED 	4
#define SB_PGP_WRITEFLG_SIGHASHTRAILER 	9
#define SB_PGP_WRITEFLG_NOSIGNATURES 	1
#define SB_PGP_WRITEFLG_WRITETRUST 	4
#define SB_PGP_MAX_SIGSTREAM_LENGTH 	65536

typedef uint8_t TSBPGPProtectionTypeRaw;

typedef enum
{
	_ptNone = 0,
	ptLow = 1,
	ptNormal = 2,
	ptHigh = 3
} TSBPGPProtectionType;

#ifdef __cplusplus
};	/* extern "C" */
#endif

void Register_SBPGPConstants_Constants(int module_number TSRMLS_DC);
void Register_SBPGPConstants_Enum_Flags(TSRMLS_D);

#ifdef __cplusplus
};	/* namespace SecureBlackbox */
#endif

#pragma pack(pop)

#endif  /* __INC_SBPGPCONSTANTS */

