#ifndef __INC_SBSRP
#define __INC_SBSRP

#if _MSC_VER > 1000
#  pragma once
#endif // _MSC_VER > 1000

#include "sbdefs.h"
#include "sbphpcore.h"
#include "sbsystem.h"
#include "sbtypes.h"
#include "sbutils.h"
#include "sbhashfunction.h"
#include "sbstreams.h"
#include "sbmath.h"
#include "sbstrutils.h"
#include "sbencoding.h"
#include "sbconstants.h"
#include "sbmd.h"

#pragma pack(push, 1)

#ifdef __cplusplus
namespace SecureBlackbox {
#endif

#ifdef __cplusplus
extern "C" {
#endif
#define SB_TSRPPrimesGen_1536 	2
#define SB_TSRPPrimesGen_2048 	2
#define SB_TSRPPrimesGen_3072 	5
#define SB_TSRPPrimesGen_4096 	5
#define SB_TSRPPrimesGen_6144 	5
#define SB_TSRPPrimesGen_8192 	19

typedef TElClassHandle TElSRPCredentialHandle;

typedef TElClassHandle TElSRPCredentialStoreHandle;

#pragma pack(4)
typedef struct 
{
	void * Salt;
	PLInt A;
	PLInt A_Small;
	PLInt B;
	PLInt B_Small;
	PLInt V;
	PLInt U;
	PLInt N;
	PLInt K;
	PLInt G;
	PLInt X;
	PLInt S;
	int8_t Initialized;
} TSRPContext;

typedef uint8_t TSSRPPrimeLenRaw;

typedef enum
{
	sr1024 = 0,
	sr1536 = 1,
	sr2048 = 2,
	sr3072 = 3,
	sr4096 = 4,
	sr6144 = 5,
	sr8192 = 6
} TSSRPPrimeLen;

typedef uint8_t TSBSRPCredentialPrimeLenRaw;

typedef enum
{
	srppl1024 = 0,
	srpp1536 = 1,
	srpp2048 = 2,
	srpp3072 = 3,
	srpp4096 = 4,
	srpp6144 = 5,
	srpp8192 = 6
} TSBSRPCredentialPrimeLen;

#ifdef SB_USE_CLASS_TELSRPCREDENTIAL
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_Generate(TElSRPCredentialHandle _Handle, const char * pcUsername, int32_t szUsername, const char * pcPassword, int32_t szPassword, TSBSRPCredentialPrimeLenRaw PrimeLen);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_Generate_1(TElSRPCredentialHandle _Handle, const char * pcUsername, int32_t szUsername, const char * pcPassword, int32_t szPassword, const uint8_t pPrime[], int32_t szPrime, const uint8_t pGenerator[], int32_t szGenerator);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_Load(TElSRPCredentialHandle _Handle, const char * pcData, int32_t szData);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_Load_1(TElSRPCredentialHandle _Handle, const char * pcUsername, int32_t szUsername, const uint8_t pSalt[], int32_t szSalt, const uint8_t pPrime[], int32_t szPrime, const uint8_t pGenerator[], int32_t szGenerator, const uint8_t pVerifier[], int32_t szVerifier);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_Save(TElSRPCredentialHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_get_Username(TElSRPCredentialHandle _Handle, char * pcOutResult, int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_get_Prime(TElSRPCredentialHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_get_Generator(TElSRPCredentialHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_set_Generator(TElSRPCredentialHandle _Handle, const uint8_t pValue[], int32_t szValue);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_get_Salt(TElSRPCredentialHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_get_Verifier(TElSRPCredentialHandle _Handle, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredential_Create(TElSRPCredentialHandle * OutResult);
#endif /* SB_USE_CLASS_TELSRPCREDENTIAL */

#ifdef SB_USE_CLASS_TELSRPCREDENTIALSTORE
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredentialStore_Add(TElSRPCredentialStoreHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredentialStore_Add_1(TElSRPCredentialStoreHandle _Handle, const char * pcUsername, int32_t szUsername, const char * pcPassword, int32_t szPassword, TSBSRPCredentialPrimeLenRaw PrimeLen, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredentialStore_Remove(TElSRPCredentialStoreHandle _Handle, int32_t Index);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredentialStore_Clear(TElSRPCredentialStoreHandle _Handle);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredentialStore_LoadFromStream(TElSRPCredentialStoreHandle _Handle, TStreamHandle Stream);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredentialStore_SaveToStream(TElSRPCredentialStoreHandle _Handle, TStreamHandle Stream);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredentialStore_get_Credentials(TElSRPCredentialStoreHandle _Handle, int32_t Index, TElSRPCredentialHandle * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredentialStore_get_Count(TElSRPCredentialStoreHandle _Handle, int32_t * OutResult);
SB_IMPORT uint32_t SB_APIENTRY TElSRPCredentialStore_Create(TComponentHandle AOwner, TElSRPCredentialStoreHandle * OutResult);
#endif /* SB_USE_CLASS_TELSRPCREDENTIALSTORE */

#ifdef __cplusplus
};	/* extern "C" */
#endif

extern zend_class_entry *TSRPContext_ce_ptr;
extern zend_class_entry *TElSRPCredential_ce_ptr;
extern zend_class_entry *TElSRPCredentialStore_ce_ptr;

void Register_TSRPContext(TSRMLS_D);
void Register_TElSRPCredential(TSRMLS_D);
void Register_TElSRPCredentialStore(TSRMLS_D);
SB_PHP_FUNCTION(SBSRP, SrpInitContext);
SB_PHP_FUNCTION(SBSRP, SrpDestroyContext);
SB_PHP_FUNCTION(SBSRP, SrpServerInit);
SB_PHP_FUNCTION(SBSRP, SrpGetU);
SB_PHP_FUNCTION(SBSRP, SrpGetUTLS);
SB_PHP_FUNCTION(SBSRP, SrpGetA);
SB_PHP_FUNCTION(SBSRP, SrpGetB);
SB_PHP_FUNCTION(SBSRP, SrpGetK);
SB_PHP_FUNCTION(SBSRP, SrpPrepareClientX);
SB_PHP_FUNCTION(SBSRP, SrpGetClientKey);
SB_PHP_FUNCTION(SBSRP, SrpPrepareClientKeyTLS);
SB_PHP_FUNCTION(SBSRP, SrpPrepareServerPreKeyTLS);
SB_PHP_FUNCTION(SBSRP, SrpPrepareServerKeyTLS);
SB_PHP_FUNCTION(SBSRP, SrpGetServerKey);
SB_PHP_FUNCTION(SBSRP, SrpGetNewServerData);
SB_PHP_FUNCTION(SBSRP, SrpLIntToBytesImplicit);
SB_PHP_FUNCTION(SBSRP, SrpLIntToBytesPAD);
SB_PHP_FUNCTION(SBSRP, LIntToBytes);
SB_PHP_FUNCTION(SBSRP, LInitBytes);
SB_PHP_FUNCTION(SBSRP, SrpGenerateVerifier);
void Register_SBSRP_Constants(int module_number TSRMLS_DC);
void Register_SBSRP_Enum_Flags(TSRMLS_D);

#ifdef __cplusplus
extern "C" {
#endif

#ifdef SB_USE_GLOBAL_PROCS_SRP
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpInitContext(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpDestroyContext(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpServerInit(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpGetU(PLInt N, PLInt A, PLInt B, const char * pcProto, int32_t szProto, PLInt * U);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpGetUTLS(PLInt A, PLInt B, PLInt N, PLInt * U);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpGetA(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpGetB(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpGetK(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpPrepareClientX(const char * pcUsername, int32_t szUsername, const char * pcPassword, int32_t szPassword, const uint8_t pSalt[], int32_t szSalt, PLInt * ClX);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpGetClientKey(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpPrepareClientKeyTLS(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpPrepareServerPreKeyTLS(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpPrepareServerKeyTLS(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpGetServerKey(TSRPContext * SRP);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpGetNewServerData(const char * pcUserName, int32_t szUserName, const char * pcUserPassword, int32_t szUserPassword, TSSRPPrimeLenRaw PrimeLen, uint8_t pN[], int32_t * szN, uint8_t pG[], int32_t * szG, uint8_t pSalt[], int32_t * szSalt, uint8_t pV[], int32_t * szV);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpLIntToBytesImplicit(PLInt X, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpLIntToBytesPAD(PLInt X, int32_t Len, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_LIntToBytes(PLInt I, uint8_t pOutResult[], int32_t * szOutResult);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_LInitBytes(PLInt I, const uint8_t pBA[], int32_t szBA);
SB_IMPORT uint32_t SB_APIENTRY SBSRP_SrpGenerateVerifier(const char * pcUsername, int32_t szUsername, const char * pcPassword, int32_t szPassword, const uint8_t pPrime[], int32_t szPrime, const uint8_t pGenerator[], int32_t szGenerator, const uint8_t pSalt[], int32_t szSalt, uint8_t pOutResult[], int32_t * szOutResult);
#endif /* SB_USE_GLOBAL_PROCS_SRP */

#ifdef __cplusplus
};	/* extern "C" */
#endif

#ifdef __cplusplus
};	/* namespace SecureBlackbox */
#endif

#pragma pack(pop)

#endif  /* __INC_SBSRP */

