<?php

    // This is a non-UI script that accepts hash signing result from the applet via POST
    // and completes the PDF signing process. This script is only triggered by the applet,
    // it is never invoked by the user.

    $log = "";

    error_reporting(E_ALL);
    date_default_timezone_set("UTC");
    
    if (!isset($_GET["sessionid"])) {
	return; 
    }

    $session_id = $_GET["sessionid"];

    // creating per-session temporary directory for the signing operation
    $path = "../work/" . $session_id;

    if (is_dir($path) === false) {
        return;
    }

    // proceeding to the pre-signing operation
    if (!extension_loaded("sbb") && !function_exists('SBUtils\SetLicenseKey')) {
        return;
    }

    // setting SecureBlackbox license key
    try {
        if (!SBUtils\GetLicense()) {

            #error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it in "sbb.license_key" php.ini directive or here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
            SBUtils\SetLicenseKey("");
        }
    } catch (SBException $e) {
        return;
    }

    // creating a copy of the pre-signed file 
    copy($path . "/presigned.pdf", $path . "/completed.pdf");

    // obtaining XML response
    $responseStateString = file_get_contents('php://input');

    $log .= ("POST received: " . strlen($responseStateString) . " bytes\r\n");

    // note that while you are OK with relative paths
    // in the code above, you need to pass absolute paths to SBB components

    $cwd = getcwd();
    $workdir = $cwd . "/../work/" . $session_id;

    // completing the signing
    try {
        $state = new TElDCAsyncState();
        $stateStream = new TMemoryStream();
        SBStreams\StreamWrite($stateStream, SBStrUtils\StrToUTF8($responseStateString));
        $stateStream->Position = 0;

        $state->LoadFromStream($stateStream, new TElDCXMLEncoding());

        $log .= "Response state loaded\r\n";

        $doc = new TElPDFDocument(null);
        $handler = new TElPDFPublicKeySecurityHandler(null);
        $handler->SignatureType = TSBPDFPublicKeySignatureType::pstPKCS7SHA1;

        $pdfStream = new TFileStream($workdir . "/completed.pdf", TFileMode::fmOpenReadWrite);

        $log .= "Pre-signed PDF document opened\r\n";

        $doc->CompleteAsyncOperation($pdfStream, $state, $handler);

        $log .= "Asynchronous operation completed\r\n";
        $log .= "Document signed successfully\r\n";

        $pdfStream->Free();

    } catch(SBException $e) {
        $error_happened = true;
        $error_details = "PDF signing failed with message: " . $e->getErrorMessage();
    } catch(Exception $e) {
        $error_happened = true;
        $error_details = "General exception: " . $e->getMessage();
    }

    if ($error_happened) {
        $log .= ("Error happened: " . $error_details . ")\r\n");
	$flag = "0";
    } else {
        $flag = "1";
    }

    // saving the log
    $f = fopen($workdir . "/log.txt", "w+");
    fwrite($f, $log);
    fclose($f);    

    // saving the status
    $f = fopen($workdir . "/status.txt", "w+");
    fwrite($f, $flag);
    fclose($f);    

?>
